# Copyright 2023 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import datetime
import os
from unittest import mock

import botocore.session
from botocore.exceptions import ClientError, ProfileNotFound

from awscli.customizations.configure.mfalogin import (
    ConfigureMFALoginCommand,
    InteractiveMFAPrompter,
)
from awscli.testutils import unittest


class TestInteractiveMFAPrompter(unittest.TestCase):
    def test_get_value_with_response(self):
        prompter = InteractiveMFAPrompter()
        # Mock the entire compat_input function, not just the return value
        with mock.patch(
            'awscli.customizations.configure.mfalogin.compat_input'
        ) as mock_input:
            mock_input.return_value = 'response'
            self.assertEqual(
                prompter.get_value('current', 'prompt'), 'response'
            )

    def test_get_value_with_no_response(self):
        prompter = InteractiveMFAPrompter()
        # Mock the entire compat_input function, not just the return value
        with mock.patch(
            'awscli.customizations.configure.mfalogin.compat_input'
        ) as mock_input:
            mock_input.return_value = ''
            self.assertEqual(
                prompter.get_value('current', 'prompt'), 'current'
            )


class TestConfigureMFALoginCommand(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock()
        self.session.get_scoped_config.return_value = {}
        self.session.get_credentials.return_value = mock.Mock()
        # Add available_profiles to the session mock
        self.session.available_profiles = ['default', 'test']
        self.prompter = mock.Mock()
        self.config_writer = mock.Mock()
        self.command = ConfigureMFALoginCommand(
            self.session,
            prompter=self.prompter,
            config_writer=self.config_writer,
        )
        self.parsed_args = mock.Mock()
        self.parsed_args.profile = None
        self.parsed_args.update_profile = None
        self.parsed_args.duration_seconds = None
        self.parsed_args.serial_number = None
        self.parsed_globals = mock.Mock()
        # Set profile in parsed_globals
        self.parsed_globals.profile = 'default'

    def test_no_credentials_found(self):
        # Setup mock responses for interactive prompting
        self.prompter.get_credential_value.side_effect = [
            'AKIAIOSFODNN7EXAMPLE',  # access key
            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY',  # secret key
            'arn:aws:iam::123456789012:mfa/user',  # MFA serial
            '123456',  # MFA token
        ]
        self.prompter.get_value.return_value = 'session-test'  # profile name
        
        # Set session to return None credentials to trigger interactive prompting
        self.session.get_credentials.return_value = None
        
        # Mock STS for the interactive prompting path
        sts_client = mock.Mock()
        sts_client.get_session_token.return_value = {
            'Credentials': {
                'AccessKeyId': 'ASIAIOSFODNN7EXAMPLE',
                'SecretAccessKey': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYzEXAMPLEKEY',
                'SessionToken': 'SESSION_TOKEN',
                'Expiration': datetime.datetime(2023, 5, 19, 18, 6, 10),
            }
        }
        
        with mock.patch('botocore.session.Session') as mock_session_class:
            mock_session_instance = mock.Mock()
            mock_session_instance.create_client.return_value = sts_client
            mock_session_class.return_value = mock_session_instance
            
            with mock.patch(
                'sys.stdin.isatty', return_value=False
            ):  # Non-interactive
                with mock.patch('os.path.expanduser', return_value='/tmp/credentials'):
                    with mock.patch('sys.stdout'):
                        rc = self.command._run_main(
                            self.parsed_args, self.parsed_globals
                        )
                        self.assertEqual(rc, 0)  # Should succeed via interactive prompting

    def test_profile_not_found(self):
        # Set profile to a non-existent profile
        self.parsed_globals.profile = 'nonexistent'
        
        # Mock the session to have no credentials for the nonexistent profile
        self.session.get_credentials.return_value = None
        self.session.get_scoped_config.return_value = {}
        
        # Setup mock responses for interactive prompting since no credentials found
        self.prompter.get_credential_value.side_effect = [
            'AKIAIOSFODNN7EXAMPLE',  # access key
            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY',  # secret key
            'arn:aws:iam::123456789012:mfa/user',  # MFA serial
            '123456',  # MFA token
        ]
        self.prompter.get_value.return_value = 'session-test'  # profile name
        
        # Mock STS for the interactive prompting path
        sts_client = mock.Mock()
        sts_client.get_session_token.return_value = {
            'Credentials': {
                'AccessKeyId': 'ASIAIOSFODNN7EXAMPLE',
                'SecretAccessKey': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYzEXAMPLEKEY',
                'SessionToken': 'SESSION_TOKEN',
                'Expiration': datetime.datetime(2023, 5, 19, 18, 6, 10),
            }
        }
        
        with mock.patch('botocore.session.Session') as mock_session_class:
            mock_session_instance = mock.Mock()
            mock_session_instance.create_client.return_value = sts_client
            mock_session_class.return_value = mock_session_instance
            
            with mock.patch('os.path.expanduser', return_value='/tmp/credentials'):
                with mock.patch('sys.stdout'):
                    rc = self.command._run_main(
                        self.parsed_args, self.parsed_globals
                    )
                    # Should succeed via interactive prompting
                    self.assertEqual(rc, 0)

    def test_no_mfa_serial_provided(self):
        # Mock botocore.session.Session
        mock_session = mock.Mock()
        mock_session.get_credentials.return_value = mock.Mock()
        mock_session.get_scoped_config.return_value = {}
        mock_session.available_profiles = ['default']

        with mock.patch('botocore.session.Session', return_value=mock_session):
            with mock.patch('sys.stdin.isatty', return_value=True):
                self.prompter.get_credential_value.return_value = None
                with mock.patch('sys.stderr') as mock_stderr:
                    rc = self.command._run_main(
                        self.parsed_args, self.parsed_globals
                    )
                    self.assertEqual(rc, 1)
                    all_writes = ''.join(str(call) for call in mock_stderr.write.call_args_list)
                    self.assertIn("aws: [ERROR]: MFA serial number or MFA device ARN is required", all_writes)

    def test_no_token_code_provided(self):
        # Mock botocore.session.Session
        mock_session = mock.Mock()
        mock_session.get_credentials.return_value = mock.Mock()
        mock_session.get_scoped_config.return_value = {}
        mock_session.available_profiles = ['default']

        with mock.patch('botocore.session.Session', return_value=mock_session):
            with mock.patch('sys.stdin.isatty', return_value=True):
                self.prompter.get_credential_value.side_effect = [
                    'arn:aws:iam::123456789012:mfa/user',
                    None,
                ]
                with mock.patch('sys.stderr') as mock_stderr:
                    rc = self.command._run_main(
                        self.parsed_args, self.parsed_globals
                    )
                    self.assertEqual(rc, 1)
                    all_writes = ''.join(str(call) for call in mock_stderr.write.call_args_list)
                    self.assertIn("aws: [ERROR]: MFA token code is required", all_writes)

    def test_sts_client_error(self):
        self.session.get_scoped_config.return_value = {}

        sts_client = mock.Mock()
        sts_client.get_session_token.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'InvalidClientTokenId',
                    'Message': 'Test error',
                }
            },
            'GetSessionToken',
        )
        self.session.create_client.return_value = sts_client

        self.prompter.get_credential_value.side_effect = [
            'arn:aws:iam::123456789012:mfa/user',
            '123456',
        ]
        self.prompter.get_value.return_value = 'session-test'

        with mock.patch('sys.stderr') as mock_stderr:
            rc = self.command._run_main(
                self.parsed_args, self.parsed_globals
            )
            self.assertEqual(rc, 1)
            all_writes = ''.join(str(call) for call in mock_stderr.write.call_args_list)
            self.assertIn("aws: [ERROR]:", all_writes)
            self.assertIn("An error occurred", all_writes)

    def test_successful_mfa_login(self):
        # Setup
        self.parsed_args.duration_seconds = 43200
        self.prompter.get_credential_value.side_effect = [
            'arn:aws:iam::123456789012:mfa/user',
            '123456',
        ]
        self.prompter.get_value.return_value = 'session-test'

        expiration = datetime.datetime(2023, 5, 19, 18, 6, 10)
        sts_response = {
            'Credentials': {
                'AccessKeyId': 'ASIAIOSFODNN7EXAMPLE',
                'SecretAccessKey': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYzEXAMPLEKEY',
                'SessionToken': 'AQoEXAMPLEH4aoAH0gNCAPyJxz4BlCFFxWNE1OPTgk5TthT+FvwqnKwRcOIfrRh3c/LTo6UDdyJwOOvEVPvLXCrrrUtdnniCEXAMPLE/IvU1dYUg2RVAJBanLiHb4IgRmpRV3zrkuWJOgQs8IZZaIv2BXIa2R4OlgkBN9bkUDNCJiBeb/AXlzBBko7b15fjrBs2+cTQtpZ3CYWFXG8C5zqx37wnOE49mRl/+OtkIKGO7fAE',
                'Expiration': expiration,
            }
        }

        sts_client = mock.Mock()
        sts_client.get_session_token.return_value = sts_response
        self.session.create_client.return_value = sts_client

        with mock.patch('sys.stdin.isatty', return_value=True):
            with mock.patch(
                'os.path.expanduser', return_value='/tmp/credentials'
            ):
                with mock.patch('sys.stdout'):
                    rc = self.command._run_main(
                        self.parsed_args, self.parsed_globals
                    )

        # Verify
        self.assertEqual(rc, 0)

        # Check STS was called correctly
        sts_client.get_session_token.assert_called_with(
            DurationSeconds=43200,
            SerialNumber='arn:aws:iam::123456789012:mfa/user',
            TokenCode='123456',
        )

        # Check config writer was called correctly
        expected_values = {
            '__section__': 'session-test',
            'aws_access_key_id': 'ASIAIOSFODNN7EXAMPLE',
            'aws_secret_access_key': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYzEXAMPLEKEY',
            'aws_session_token': 'AQoEXAMPLEH4aoAH0gNCAPyJxz4BlCFFxWNE1OPTgk5TthT+FvwqnKwRcOIfrRh3c/LTo6UDdyJwOOvEVPvLXCrrrUtdnniCEXAMPLE/IvU1dYUg2RVAJBanLiHb4IgRmpRV3zrkuWJOgQs8IZZaIv2BXIa2R4OlgkBN9bkUDNCJiBeb/AXlzBBko7b15fjrBs2+cTQtpZ3CYWFXG8C5zqx37wnOE49mRl/+OtkIKGO7fAE',
        }

        self.config_writer.update_config.assert_called_with(
            expected_values, '/tmp/credentials', check_permissions=True
        )

    def test_serial_number_from_parameter(self):
        # Setup - use serial number from parameter
        self.parsed_args.serial_number = (
            'arn:aws:iam::123456789012:mfa/user-param'
        )
        self.parsed_args.duration_seconds = 43200

        self.session.get_scoped_config.return_value = {
            'mfa_serial': 'arn:aws:iam::123456789012:mfa/user-config'
        }

        sts_client = mock.Mock()
        expiration = datetime.datetime(2023, 5, 19, 18, 6, 10)
        sts_response = {
            'Credentials': {
                'AccessKeyId': 'ASIAIOSFODNN7EXAMPLE',
                'SecretAccessKey': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYzEXAMPLEKEY',
                'SessionToken': 'SESSION_TOKEN',
                'Expiration': expiration,
            }
        }
        sts_client.get_session_token.return_value = sts_response
        self.session.create_client.return_value = sts_client

        with mock.patch('sys.stdin.isatty', return_value=True):
            self.prompter.get_credential_value.return_value = '123456'
            self.prompter.get_value.return_value = 'session-test'
            with mock.patch(
                'os.path.expanduser', return_value='/tmp/credentials'
            ):
                with mock.patch('sys.stdout'):
                    rc = self.command._run_main(
                        self.parsed_args, self.parsed_globals
                    )

        # Verify
        self.assertEqual(rc, 0)

        # Check that the parameter value was used instead of the config value
        sts_client.get_session_token.assert_called_with(
            DurationSeconds=43200,
            SerialNumber='arn:aws:iam::123456789012:mfa/user-param',
            TokenCode='123456',
        )

    def test_missing_default_profile_interactive(self):
        """Test prompting for credentials when no default profile exists in interactive mode."""
        self.parsed_globals.profile = None  # Use default profile
        
        # Set session to return None credentials to trigger interactive prompting
        self.session.get_credentials.return_value = None

        # Mock sys.stdin.isatty to return True (interactive)
        with mock.patch('sys.stdin.isatty', return_value=True):
            # Mock the _handle_interactive_prompting method
            with mock.patch.object(
                self.command,
                '_handle_interactive_prompting',
                return_value=0,
            ) as mock_handle:
                rc = self.command._run_main(
                    self.parsed_args, self.parsed_globals
                )

                self.assertEqual(rc, 0)
                mock_handle.assert_called_once_with(
                    self.parsed_args, None
                )

    def test_missing_default_profile_non_interactive(self):
        """Test error when no default profile exists in non-interactive mode."""
        self.parsed_globals.profile = None  # Use default profile
        
        # Setup mock responses for interactive prompting
        self.prompter.get_credential_value.side_effect = [
            'AKIAIOSFODNN7EXAMPLE',  # access key
            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY',  # secret key
            'arn:aws:iam::123456789012:mfa/user',  # MFA serial
            '123456',  # MFA token
        ]
        self.prompter.get_value.return_value = 'session-test'  # profile name

        # Set session to return None credentials to trigger interactive prompting
        self.session.get_credentials.return_value = None
        
        # Mock STS for the interactive prompting path
        sts_client = mock.Mock()
        sts_client.get_session_token.return_value = {
            'Credentials': {
                'AccessKeyId': 'ASIAIOSFODNN7EXAMPLE',
                'SecretAccessKey': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYzEXAMPLEKEY',
                'SessionToken': 'SESSION_TOKEN',
                'Expiration': datetime.datetime(2023, 5, 19, 18, 6, 10),
            }
        }
        
        with mock.patch('botocore.session.Session') as mock_session_class:
            mock_session_instance = mock.Mock()
            mock_session_instance.create_client.return_value = sts_client
            mock_session_class.return_value = mock_session_instance
            
            with mock.patch('sys.stdin.isatty', return_value=False):
                with mock.patch('os.path.expanduser', return_value='/tmp/credentials'):
                    with mock.patch('sys.stdout'):
                        rc = self.command._run_main(
                            self.parsed_args, self.parsed_globals
                        )

                        self.assertEqual(rc, 0)  # Should succeed via interactive prompting

    def test_handle_missing_default_profile_success(self):
        """Test successful credential prompting and MFA login when no default profile exists."""
        # Setup mock responses for prompting - now all via get_credential_value
        self.prompter.get_credential_value.side_effect = [
            'AKIAIOSFODNN7EXAMPLE',  # access key
            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY',  # secret key
            'arn:aws:iam::123456789012:mfa/user',  # MFA serial
            '123456',  # MFA token
        ]
        self.prompter.get_value.return_value = 'session-test'  # profile name

        # Mock STS response
        expiration = datetime.datetime(2023, 5, 19, 18, 6, 10)
        sts_response = {
            'Credentials': {
                'AccessKeyId': 'ASIAIOSFODNN7EXAMPLE',
                'SecretAccessKey': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYzEXAMPLEKEY',
                'SessionToken': 'SESSION_TOKEN',
                'Expiration': expiration,
            }
        }

        # Mock botocore session and STS client
        mock_session = mock.Mock()
        sts_client = mock.Mock()
        sts_client.get_session_token.return_value = sts_response
        mock_session.create_client.return_value = sts_client

        with mock.patch('botocore.session.Session', return_value=mock_session):
            with mock.patch(
                'os.path.expanduser', return_value='/tmp/credentials'
            ):
                with mock.patch('sys.stdout'):
                    rc = self.command._handle_interactive_prompting(
                        self.parsed_args, 43200
                    )

        # Verify success
        self.assertEqual(rc, 0)

        # Verify STS call
        sts_client.get_session_token.assert_called_with(
            DurationSeconds=43200,
            SerialNumber='arn:aws:iam::123456789012:mfa/user',
            TokenCode='123456',
        )

        # Verify only the session profile was written
        self.assertEqual(self.config_writer.update_config.call_count, 1)

    def test_handle_missing_default_profile_missing_access_key(self):
        """Test error when access key is not provided."""
        self.prompter.get_credential_value.return_value = (
            None  # No access key provided
        )

        with mock.patch('sys.stderr') as mock_stderr:
            rc = self.command._handle_interactive_prompting(
                self.parsed_args, 43200
            )

            self.assertEqual(rc, 1)
            all_writes = ''.join(str(call) for call in mock_stderr.write.call_args_list)
            self.assertIn("aws: [ERROR]: AWS Access Key ID is required", all_writes)

    def test_handle_missing_default_profile_missing_secret_key(self):
        """Test error when secret key is not provided."""
        self.prompter.get_credential_value.side_effect = [
            'AKIAIOSFODNN7EXAMPLE',  # access key provided
            None,  # secret key not provided
        ]

        with mock.patch('sys.stderr') as mock_stderr:
            rc = self.command._handle_interactive_prompting(
                self.parsed_args, 43200
            )

            self.assertEqual(rc, 1)
            all_writes = ''.join(str(call) for call in mock_stderr.write.call_args_list)
            self.assertIn("aws: [ERROR]: AWS Secret Access Key is required", all_writes)

    def test_credential_value_prompting_clean_display(self):
        """Test that credential prompting doesn't show default values."""
        prompter = InteractiveMFAPrompter()
        with mock.patch(
            'awscli.customizations.configure.mfalogin.compat_input'
        ) as mock_input:
            mock_input.return_value = 'test-value'
            result = prompter.get_credential_value(
                'None', 'aws_access_key_id', 'AWS Access Key ID'
            )

            # Verify the prompt doesn't show [None] or any default value
            mock_input.assert_called_with('AWS Access Key ID: ')
            self.assertEqual(result, 'test-value')

    def test_handle_missing_default_profile_sts_error(self):
        """Test STS error handling in missing default profile scenario."""
        self.prompter.get_credential_value.side_effect = [
            'AKIAIOSFODNN7EXAMPLE',  # access key
            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY',  # secret key
            'arn:aws:iam::123456789012:mfa/user',  # MFA serial
            '123456',  # MFA token
        ]
        self.prompter.get_value.return_value = 'session-test'  # profile name

        # Mock STS client to raise an error
        mock_session = mock.Mock()
        sts_client = mock.Mock()
        sts_client.get_session_token.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'InvalidClientTokenId',
                    'Message': 'Invalid credentials',
                }
            },
            'GetSessionToken',
        )
        mock_session.create_client.return_value = sts_client

        with mock.patch('botocore.session.Session', return_value=mock_session):
            with mock.patch('sys.stderr') as mock_stderr:
                rc = self.command._handle_interactive_prompting(
                    self.parsed_args, 43200
                )

                self.assertEqual(rc, 1)
                all_writes = ''.join(str(call) for call in mock_stderr.write.call_args_list)
                self.assertIn("aws: [ERROR]:", all_writes)
                self.assertIn("An error occurred", all_writes)

    def test_non_interactive_missing_mfa_serial(self):
        """Test non-interactive mode when MFA serial is missing."""
        mock_session = mock.Mock()
        mock_session.get_credentials.return_value = mock.Mock()
        mock_session.get_scoped_config.return_value = {}  # No mfa_serial in config
        mock_session.available_profiles = ['default']

        with mock.patch('botocore.session.Session', return_value=mock_session):
            with mock.patch(
                'sys.stdin.isatty', return_value=False
            ):  # Non-interactive
                self.prompter.get_credential_value.return_value = None
                with mock.patch('sys.stderr') as mock_stderr:
                    rc = self.command._run_main(
                        self.parsed_args, self.parsed_globals
                    )

                    self.assertEqual(rc, 1)
                    all_writes = ''.join(str(call) for call in mock_stderr.write.call_args_list)
                    self.assertIn("aws: [ERROR]: MFA serial number or MFA device ARN is required", all_writes)

    def test_non_interactive_missing_token_code(self):
        """Test non-interactive mode when token code would be prompted."""
        self.session.get_scoped_config.return_value = {
            'mfa_serial': 'arn:aws:iam::123456789012:mfa/user'
        }

        with mock.patch(
            'sys.stdin.isatty', return_value=False
        ):  # Non-interactive
            self.prompter.get_credential_value.return_value = None
            with mock.patch('sys.stderr') as mock_stderr:
                rc = self.command._run_main(
                    self.parsed_args, self.parsed_globals
                )

                self.assertEqual(rc, 1)
                all_writes = ''.join(str(call) for call in mock_stderr.write.call_args_list)
                self.assertIn("aws: [ERROR]: MFA token code is required", all_writes)

    def test_empty_credential_input_handling(self):
        """Test handling of empty credential inputs."""
        self.prompter.get_credential_value.return_value = ''  # Empty string

        with mock.patch('sys.stderr') as mock_stderr:
            rc = self.command._handle_interactive_prompting(
                self.parsed_args, 43200
            )

            self.assertEqual(rc, 1)
            all_writes = ''.join(str(call) for call in mock_stderr.write.call_args_list)
            self.assertIn("aws: [ERROR]: AWS Access Key ID is required", all_writes)

    def test_check_permissions_when_writing_temporary_credentials(self):
        self.prompter.get_credential_value.side_effect = [
            'arn:aws:iam::123456789012:mfa/user',
            '123456',
        ]
        self.prompter.get_value.return_value = 'session-test'

        expiration = datetime.datetime(2023, 5, 19, 18, 6, 10)
        sts_response = {
            'Credentials': {
                'AccessKeyId': 'ASIAIOSFODNN7EXAMPLE',
                'SecretAccessKey': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYzEXAMPLEKEY',
                'SessionToken': 'SESSION_TOKEN',
                'Expiration': expiration,
            }
        }

        sts_client = mock.Mock()
        sts_client.get_session_token.return_value = sts_response
        self.session.create_client.return_value = sts_client

        with mock.patch('sys.stdin.isatty', return_value=True):
            with mock.patch(
                'os.path.expanduser', return_value='/tmp/credentials'
            ):
                with mock.patch('sys.stdout'):
                    rc = self.command._run_main(
                        self.parsed_args, self.parsed_globals
                    )

        self.assertEqual(rc, 0)
        self.config_writer.update_config.assert_called_once_with(
            mock.ANY, '/tmp/credentials', check_permissions=True
        )



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest, FileCreator


class BaseRekognitionTest(BaseAWSCommandParamsTest):
    def setUp(self):
        super(BaseRekognitionTest, self).setUp()
        self.files = FileCreator()
        self.temp_file = self.files.create_file('foo', 'mycontents')
        with open(self.temp_file, 'rb') as f:
            self.temp_file_bytes = f.read()

    def tearDown(self):
        super(BaseRekognitionTest, self).tearDown()
        self.files.remove_all()


class TestCompareFaces(BaseRekognitionTest):
    prefix = 'rekognition compare-faces'

    def test_image_file_does_populate_bytes_param(self):
        second_temp_file = self.files.create_file('bar', 'othercontents')
        second_file_bytes = open(second_temp_file, 'rb').read()

        cmdline = self.prefix
        cmdline += ' --source-image-bytes fileb://%s' % self.temp_file
        cmdline += ' --target-image-bytes fileb://%s' % second_temp_file
        result = {
            'SourceImage': {'Bytes': self.temp_file_bytes},
            'TargetImage': {'Bytes': second_file_bytes},
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_image_bytes_still_works(self):
        cmdline = self.prefix
        cmdline += ' --source-image Bytes=Zm9v'
        cmdline += ' --target-image Bytes=YmFy'
        result = {
            'SourceImage': {'Bytes': b'foo'},
            'TargetImage': {'Bytes': b'bar'},
        }
        self.assert_params_for_cmd(cmdline, result)


class TestDetectFaces(BaseRekognitionTest):
    prefix = 'rekognition detect-faces'

    def test_image_file_does_populate_bytes_param(self):
        cmdline = self.prefix
        cmdline += ' --image-bytes fileb://%s' % self.temp_file
        result = {'Image': {'Bytes': self.temp_file_bytes}}
        self.assert_params_for_cmd(cmdline, result)

    def test_image_bytes_still_works(self):
        cmdline = self.prefix
        cmdline += ' --image Bytes=Zm9vYmFy'
        result = {'Image': {'Bytes': b'foobar'}}
        self.assert_params_for_cmd(cmdline, result)


class TestDetectLabels(BaseRekognitionTest):
    prefix = 'rekognition detect-labels'

    def test_image_file_does_populate_bytes_param(self):
        cmdline = self.prefix
        cmdline += ' --image-bytes fileb://%s' % self.temp_file
        result = {'Image': {'Bytes': self.temp_file_bytes}}
        self.assert_params_for_cmd(cmdline, result)

    def test_image_bytes_still_works(self):
        cmdline = self.prefix
        cmdline += ' --image Bytes=Zm9vYmFy'
        result = {'Image': {'Bytes': b'foobar'}}
        self.assert_params_for_cmd(cmdline, result)


class TestDetectModerationLabels(BaseRekognitionTest):
    prefix = 'rekognition detect-moderation-labels'

    def test_image_file_does_populate_bytes_param(self):
        cmdline = self.prefix
        cmdline += ' --image-bytes fileb://%s' % self.temp_file
        result = {'Image': {'Bytes': self.temp_file_bytes}}
        self.assert_params_for_cmd(cmdline, result)

    def test_image_bytes_still_works(self):
        cmdline = self.prefix
        cmdline += ' --image Bytes=Zm9vYmFy'
        result = {'Image': {'Bytes': b'foobar'}}
        self.assert_params_for_cmd(cmdline, result)


class TestDetectText(BaseRekognitionTest):
    prefix = 'rekognition detect-text'

    def test_image_file_does_populate_bytes_param(self):
        cmdline = self.prefix
        cmdline += ' --image-bytes fileb://%s' % self.temp_file
        result = {'Image': {'Bytes': self.temp_file_bytes}}
        self.assert_params_for_cmd(cmdline, result)

    def test_image_bytes_still_works(self):
        cmdline = self.prefix
        cmdline += ' --image Bytes=Zm9vYmFy'
        result = {'Image': {'Bytes': b'foobar'}}
        self.assert_params_for_cmd(cmdline, result)


class TestIndexFaces(BaseRekognitionTest):
    prefix = 'rekognition index-faces'

    def test_image_file_does_populate_bytes_param(self):
        cmdline = self.prefix
        cmdline += ' --collection-id foobar'
        cmdline += ' --image-bytes fileb://%s' % self.temp_file
        result = {
            'CollectionId': 'foobar',
            'Image': {'Bytes': self.temp_file_bytes},
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_image_bytes_still_works(self):
        cmdline = self.prefix
        cmdline += ' --collection-id foobar'
        cmdline += ' --image Bytes=Zm9vYmFy'
        result = {'CollectionId': 'foobar', 'Image': {'Bytes': b'foobar'}}
        self.assert_params_for_cmd(cmdline, result)


class TestRecognizeCelebrities(BaseRekognitionTest):
    prefix = 'rekognition recognize-celebrities'

    def test_image_file_does_populate_bytes_param(self):
        cmdline = self.prefix
        cmdline += ' --image-bytes fileb://%s' % self.temp_file
        result = {'Image': {'Bytes': self.temp_file_bytes}}
        self.assert_params_for_cmd(cmdline, result)

    def test_image_bytes_still_works(self):
        cmdline = self.prefix
        cmdline += ' --image Bytes=Zm9vYmFy'
        result = {'Image': {'Bytes': b'foobar'}}
        self.assert_params_for_cmd(cmdline, result)


class TestSearchFacesByImage(BaseRekognitionTest):
    prefix = 'rekognition search-faces-by-image'

    def test_image_file_does_populate_bytes_param(self):
        cmdline = self.prefix
        cmdline += ' --collection-id foobar'
        cmdline += ' --image-bytes fileb://%s' % self.temp_file
        result = {
            'CollectionId': 'foobar',
            'Image': {'Bytes': self.temp_file_bytes},
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_image_bytes_still_works(self):
        cmdline = self.prefix
        cmdline += ' --collection-id foobar'
        cmdline += ' --image Bytes=Zm9vYmFy'
        result = {'CollectionId': 'foobar', 'Image': {'Bytes': b'foobar'}}
        self.assert_params_for_cmd(cmdline, result)



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0e
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import copy

from awscli.customizations import s3errormsg
from awscli.testutils import unittest


class TestGetRegionFromEndpoint(unittest.TestCase):
    def test_sigv4_error_message(self):
        parsed = {'Error': {'Message': 'Please use AWS4-HMAC-SHA256'}}
        s3errormsg.enhance_error_msg(parsed)
        # We should say how to fix the issue.
        self.assertIn('You can fix this issue', parsed['Error']['Message'])
        # We should mention the --region argument.
        self.assertIn('--region', parsed['Error']['Message'])
        # We should mention get-bucket-location
        self.assertIn('get-bucket-location', parsed['Error']['Message'])

    def test_301_error_message(self):
        parsed = {
            'Error': {
                'Code': 'PermanentRedirect',
                'Message': "Please use the correct endpoint.",
                'Endpoint': "myendpoint",
            }
        }
        s3errormsg.enhance_error_msg(parsed)
        # We should include the endpoint in the error message.
        error_message = parsed['Error']['Message']
        self.assertIn('myendpoint', error_message)

    def test_kms_sigv4_error_message(self):
        parsed = {
            'Error': {
                'Message': (
                    'Requests specifying Server Side Encryption with '
                    'AWS KMS managed keys require AWS Signature Version 4.'
                )
            }
        }
        s3errormsg.enhance_error_msg(parsed)
        # We should say how you enable it.
        self.assertIn(
            'You can enable AWS Signature Version 4',
            parsed['Error']['Message'],
        )
        # We should mention the command that needs to be run.
        self.assertIn(
            'aws configure set s3.signature_version s3v4',
            parsed['Error']['Message'],
        )

    def test_error_message_not_enhanced(self):
        parsed = {
            'Error': {
                'Message': 'This is a different error message.',
                'Code': 'Other Message',
            }
        }
        expected = copy.deepcopy(parsed)
        s3errormsg.enhance_error_msg(parsed)
        # Nothing should have changed
        self.assertEqual(parsed, expected)

    def test_not_an_error_message(self):
        parsed = {'Success': 'response', 'ResponseMetadata': {}}
        expected = copy.deepcopy(parsed)
        s3errormsg.enhance_error_msg(parsed)
        # Nothing should have changed
        self.assertEqual(parsed, expected)



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.customizations.configservice.subscribe import S3BucketHelper
from awscli.testutils import BaseAWSCommandParamsTest, mock


class TestSubscribe(BaseAWSCommandParamsTest):
    prefix = 'configservice subscribe'

    def setUp(self):
        super(TestSubscribe, self).setUp()
        self.parsed_responses = [
            {},  # S3 HeadBucket
            {'TopicArn': 'my-topic-arn'},  # SNS CreateTopic
            {},  # PutConfigurationRecorder
            {},  # PutDeliveryChannel
            {},  # StartConfigurationRecorder
            {'ConfigurationRecorders': {}},  # DescribeConfigurationRecorders
            {'DeliveryChannels': {}},  # DescribeDeliveryChannels
        ]

    def test_subscribe_when_bucket_exists_and_new_sns_topic(self):
        self.prefix += ' --s3-bucket mybucket --sns-topic mytopic'
        self.prefix += ' --iam-role myrole'
        self.run_cmd(self.prefix)

        self.assertEqual(len(self.operations_called), 7)
        list_of_operation_names_called = []
        list_of_parameters_called = []
        for operation_called in self.operations_called:
            list_of_operation_names_called.append(operation_called[0].name)
            list_of_parameters_called.append(operation_called[1])

        self.assertEqual(
            list_of_operation_names_called,
            [
                'HeadBucket',
                'CreateTopic',
                'PutConfigurationRecorder',
                'PutDeliveryChannel',
                'StartConfigurationRecorder',
                'DescribeConfigurationRecorders',
                'DescribeDeliveryChannels',
            ],
        )
        self.assertEqual(
            list_of_parameters_called,
            [
                {'Bucket': 'mybucket'},  # S3 HeadBucket
                {'Name': 'mytopic'},  # SNS CreateTopic
                {
                    'ConfigurationRecorder': {  # PutConfigurationRecorder
                        'name': 'default',
                        'roleARN': 'myrole',
                    }
                },
                {
                    'DeliveryChannel': {  # PutDeliveryChannel
                        'name': 'default',
                        's3BucketName': 'mybucket',
                        'snsTopicARN': 'my-topic-arn',
                    }
                },
                # StartConfigurationRecorder
                {'ConfigurationRecorderName': 'default'},
                {},  # DescribeConfigurationRecorders
                {},  # DescribeDeliveryChannels
            ],
        )

    def test_subscribe_when_bucket_exists_and_sns_topic_arn_provided(self):
        self.parsed_responses.pop(1)
        self.prefix += ' --s3-bucket mybucket --sns-topic arn:mytopic'
        self.prefix += ' --iam-role myrole'
        self.run_cmd(self.prefix)

        self.assertEqual(len(self.operations_called), 6)
        list_of_operation_names_called = []
        list_of_parameters_called = []
        for operation_called in self.operations_called:
            list_of_operation_names_called.append(operation_called[0].name)
            list_of_parameters_called.append(operation_called[1])

        self.assertEqual(
            list_of_operation_names_called,
            [
                'HeadBucket',
                'PutConfigurationRecorder',
                'PutDeliveryChannel',
                'StartConfigurationRecorder',
                'DescribeConfigurationRecorders',
                'DescribeDeliveryChannels',
            ],
        )
        self.assertEqual(
            list_of_parameters_called,
            [
                {'Bucket': 'mybucket'},  # S3 HeadBucket
                {
                    'ConfigurationRecorder': {  # PutConfigurationRecorder
                        'name': 'default',
                        'roleARN': 'myrole',
                    }
                },
                {
                    'DeliveryChannel': {  # PutDeliveryChannel
                        'name': 'default',
                        's3BucketName': 'mybucket',
                        'snsTopicARN': 'arn:mytopic',
                    }
                },
                # StartConfigurationRecorder
                {'ConfigurationRecorderName': 'default'},
                {},  # DescribeConfigurationRecorders
                {},  # DescribeDeliveryChannels
            ],
        )

    def test_subscribe_when_bucket_needs_to_be_created(self):
        # TODO: fix this patch when we have a better way to stub out responses
        with mock.patch(
            'botocore.endpoint.Endpoint._send'
        ) as http_session_send_patch:
            # Mock for HeadBucket request
            head_bucket_response = mock.Mock()
            head_bucket_response.status_code = 404
            head_bucket_response.content = b''
            head_bucket_response.headers = {}

            # Mock for CreateBucket request
            create_bucket_response = mock.Mock()
            create_bucket_response.status_code = 200
            create_bucket_response.content = b''
            create_bucket_response.headers = {}

            http_session_send_patch.side_effect = [
                head_bucket_response,
                create_bucket_response,
            ]

            s3_client = self.driver.session.create_client('s3')
            bucket_helper = S3BucketHelper(s3_client)
            bucket_helper.prepare_bucket('mybucket')
            send_call_list = http_session_send_patch.call_args_list
            self.assertEqual(send_call_list[0][0][0].method, 'HEAD')
            # Since the HeadObject fails with 404, the CreateBucket which is
            # is a PUT request should be made.
            self.assertEqual(send_call_list[1][0][0].method, 'PUT')



# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os
import threading
import webbrowser

import pytest
import urllib3
from botocore.exceptions import PendingAuthorizationExpiredError
from botocore.session import Session

from awscli.compat import BytesIO, StringIO
from awscli.customizations.sso.utils import (
    AuthCodeFetcher,
    OAuthCallbackHandler,
    OpenBrowserHandler,
    PrintOnlyHandler,
    do_sso_login,
    open_browser_with_original_ld_path,
    parse_sso_registration_scopes,
)
from awscli.testutils import mock, unittest


@pytest.mark.parametrize(
    'raw_scopes, parsed_scopes',
    [
        ('scope', ['scope']),
        (' scope ', ['scope']),
        ('', []),
        ('scope, ', ['scope']),
        ('scope-1,scope-2', ['scope-1', 'scope-2']),
        ('scope-1, scope-2', ['scope-1', 'scope-2']),
        (' scope-1, scope-2 ', ['scope-1', 'scope-2']),
        ('scope-1,scope-2,scope-3', ['scope-1', 'scope-2', 'scope-3']),
    ],
)
def test_parse_registration_scopes(raw_scopes, parsed_scopes):
    assert parse_sso_registration_scopes(raw_scopes) == parsed_scopes


class TestDoSSOLogin(unittest.TestCase):
    def setUp(self):
        self.region = 'us-west-2'
        self.start_url = 'https://mystarturl.com'
        self.token_cache = {}
        self.on_pending_authorization_mock = mock.Mock()
        self.session = mock.Mock(Session)
        self.oidc_client = self.get_mock_sso_oidc_client()
        self.session.create_client.return_value = self.oidc_client

    def get_mock_sso_oidc_client(self):
        real_client = Session().create_client(
            'sso-oidc', region_name=self.region
        )
        client = mock.Mock()
        client.exceptions = real_client.exceptions
        client.register_client.return_value = {
            'clientSecretExpiresAt': 1000,
            'clientId': 'foo-client-id',
            'clientSecret': 'foo-client-secret',
        }
        client.start_device_authorization.return_value = {
            'interval': 1,
            'expiresIn': 600,
            'userCode': 'foo',
            'deviceCode': 'foo-device-code',
            'verificationUri': 'https://sso.fake/device',
            'verificationUriComplete': 'https://sso.verify',
        }
        client.create_token.side_effect = [
            client.exceptions.AuthorizationPendingException({}, "CreateToken"),
            {
                'expiresIn': 28800,
                'tokenType': 'Bearer',
                'accessToken': 'access.token',
            },
        ]
        return client

    def assert_client_called_with_start_url(self):
        call_args = self.oidc_client.start_device_authorization.call_args
        self.assertEqual(call_args[1]['startUrl'], self.start_url)

    def assert_used_sso_region(self):
        config = self.session.create_client.call_args[1]['config']
        self.assertEqual(config.region_name, self.region)

    def assert_token_cache_was_filled(self):
        self.assertGreater(len(self.token_cache), 0)

    def assert_on_pending_authorization_called(self):
        self.assertEqual(
            len(self.on_pending_authorization_mock.call_args_list), 1
        )

    def test_do_sso_login(self):
        do_sso_login(
            session=self.session,
            sso_region=self.region,
            parsed_globals=mock.Mock(),
            start_url=self.start_url,
            token_cache=self.token_cache,
            on_pending_authorization=self.on_pending_authorization_mock,
        )
        # We just want to make some quick checks to make sure all of the
        # parameters were plumbed in correctly.
        self.assert_client_called_with_start_url()
        self.assert_used_sso_region()
        self.assert_token_cache_was_filled()
        self.assert_on_pending_authorization_called()

    def test_do_sso_login_preauthorized(self):
        # First call to create token succeeds because client is pre-authorized
        self.oidc_client.create_token.side_effect = [
            {
                'expiresIn': 28800,
                'tokenType': 'Bearer',
                'accessToken': 'access.token',
            }
        ]
        do_sso_login(
            session=self.session,
            sso_region=self.region,
            parsed_globals=mock.Mock(),
            start_url=self.start_url,
            token_cache=self.token_cache,
            on_pending_authorization=self.on_pending_authorization_mock,
        )
        self.assert_client_called_with_start_url()
        self.assert_used_sso_region()
        self.assert_token_cache_was_filled()
        # Handler should not have been invoked as client was pre-authorized
        self.on_pending_authorization_mock.assert_not_called()


class BaseHandlerTest(unittest.TestCase):
    def setUp(self):
        self.stream = StringIO()
        self.user_code = '12345'
        self.verification_uri = 'https://verification.com'
        self.verification_uri_complete = 'https://verification.com?code=12345'
        self.pending_authorization = {
            'userCode': self.user_code,
            'verificationUri': self.verification_uri,
            'verificationUriComplete': self.verification_uri_complete,
            'cross_device_flag': '--use-device-code',
        }

    def assert_text_in_output(self, *args):
        output = self.stream.getvalue()
        for text in args:
            self.assertIn(text, output)


class TestPrintOnlyHandler(BaseHandlerTest):
    def test_prints_message(self):
        handler = PrintOnlyHandler(self.stream)
        handler(**self.pending_authorization)
        self.assert_text_in_output(
            'Browser will not be automatically opened.',
            self.user_code,
            self.verification_uri,
            self.verification_uri_complete,
        )


class TestOpenBrowserHandler(BaseHandlerTest):
    def setUp(self):
        super().setUp()
        self.open_browser = mock.Mock(spec=webbrowser.open_new_tab)
        self.handler = OpenBrowserHandler(
            self.stream,
            open_browser=self.open_browser,
        )

    def test_call_no_browser(self):
        handler = OpenBrowserHandler(self.stream, open_browser=False)
        handler(**self.pending_authorization)
        self.assert_text_in_output(self.user_code, self.verification_uri)

    def test_call_browser_success(self):
        self.handler(**self.pending_authorization)
        self.open_browser.assert_called_with(self.verification_uri_complete)
        self.assert_text_in_output('Attempting', 'open')
        # assert the URI and user coe are still displayed
        self.assert_text_in_output(self.user_code, self.verification_uri)

    def test_call_browser_fails(self):
        self.open_browser.side_effect = webbrowser.Error()
        self.handler(**self.pending_authorization)
        self.assert_text_in_output(self.user_code, self.verification_uri)
        self.open_browser.assert_called_with(self.verification_uri_complete)


class TestOpenBrowserWithPatchedEnv(unittest.TestCase):
    def test_can_patch_env(self):
        # The various edge case are tested in original_ld_library_path,
        # we're just checking that we're integrating everything together
        # correctly.
        env = {'LD_LIBRARY_PATH': '/foo'}
        with mock.patch('os.environ', env):
            with mock.patch('webbrowser.open_new_tab') as open_new_tab:
                captured_env = {}
                open_new_tab.side_effect = lambda x: captured_env.update(
                    os.environ
                )
                open_browser_with_original_ld_path('http://example.com')
        self.assertIsNone(captured_env.get('LD_LIBRARY_PATH'))


class MockRequest:
    def __init__(self, request):
        self._request = request

    def makefile(self, *args, **kwargs):
        return BytesIO(self._request)

    def sendall(self, data):
        pass


class TestOAuthCallbackHandler:
    """Tests for OAuthCallbackHandler, which handles
    individual requests that we receive at the callback uri
    """

    def test_expected_query_params(self):
        fetcher = mock.Mock(AuthCodeFetcher)

        OAuthCallbackHandler(
            fetcher,
            MockRequest(b'GET /?state=123&code=456'),
            mock.MagicMock(),
            mock.MagicMock(),
        )
        fetcher.set_auth_code_and_state.assert_called_once_with('456', '123')

    def test_error(self):
        fetcher = mock.Mock(AuthCodeFetcher)

        OAuthCallbackHandler(
            fetcher,
            MockRequest(b'GET /?error=Error%20message'),
            mock.MagicMock(),
            mock.MagicMock(),
        )

        fetcher.set_auth_code_and_state.assert_called_once_with(None, None)

    def test_missing_expected_query_params(self):
        fetcher = mock.Mock(AuthCodeFetcher)

        # We generally don't expect to be missing the expected query params,
        # but if we do we expect the server to keep waiting for a valid callback
        OAuthCallbackHandler(
            fetcher,
            MockRequest(b'GET /'),
            mock.MagicMock(),
            mock.MagicMock(),
        )

        fetcher.set_auth_code_and_state.assert_not_called()


class TestAuthCodeFetcher:
    """Tests for the AuthCodeFetcher class, which is the local
    web server we use to handle the OAuth 2.0 callback
    """

    def setup_method(self):
        self.fetcher = AuthCodeFetcher()
        self.url = (
            f'http://127.0.0.1:{self.fetcher.http_server.server_address[1]}/'
        )

        # Start the server on a background thread so that
        # the test thread can make the request
        self.server_thread = threading.Thread(
            target=self.fetcher.get_auth_code_and_state
        )
        self.server_thread.daemon = True
        self.server_thread.start()

    def test_expected_auth_code(self):
        expected_code = '1234'
        expected_state = '4567'
        url = self.url + f'?code={expected_code}&state={expected_state}'

        http = urllib3.PoolManager()
        response = http.request("GET", url)

        actual_code, actual_state = self.fetcher.get_auth_code_and_state()
        assert response.status == 200
        assert actual_code == expected_code
        assert actual_state == expected_state

    def test_error(self):
        expected_code = 'Failed'
        url = self.url + f'?error={expected_code}'

        http = urllib3.PoolManager()
        response = http.request("GET", url)

        actual_code, actual_state = self.fetcher.get_auth_code_and_state()
        assert response.status == 200
        assert actual_code is None
        assert actual_state is None


@mock.patch(
    'awscli.customizations.sso.utils.AuthCodeFetcher._REQUEST_TIMEOUT', 0.1
)
@mock.patch(
    'awscli.customizations.sso.utils.AuthCodeFetcher._OVERALL_TIMEOUT', 0.1
)
def test_get_auth_code_and_state_timeout():
    """Tests the timeout case separately of TestAuthCodeFetcher,
    since we need to override the constants
    """
    with pytest.raises(PendingAuthorizationExpiredError):
        AuthCodeFetcher().get_auth_code_and_state()



# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestDescribeSnapshots(BaseAWSCommandParamsTest):
    prefix = 'ec2 describe-snapshots'

    def test_max_results_set_by_default(self):
        command = self.prefix
        params = {'MaxResults': 1000}
        self.assert_params_for_cmd(command, params)

    def test_max_results_not_set_with_snapshot_ids(self):
        command = self.prefix + ' --snapshot-ids snap-example'
        params = {'SnapshotIds': ['snap-example']}
        self.assert_params_for_cmd(command, params)

    def test_max_results_not_set_with_filter(self):
        command = self.prefix + ' --filters Name=snapshot-id,Values=snap-snap'
        params = {
            'Filters': [{'Name': 'snapshot-id', 'Values': ['snap-snap']}]
        }
        self.assert_params_for_cmd(command, params)

    def test_max_results_not_overwritten(self):
        command = self.prefix + ' --max-results 5'
        params = {'MaxResults': 5}
        self.assert_params_for_cmd(command, params)

        command = self.prefix + ' --page-size 5'
        self.assert_params_for_cmd(command, params)



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os

from awscli.testutils import BaseAWSCommandParamsTest, FileCreator
from awscli.utils import dump_yaml_to_str


class BaseCLIInputArgumentTest(BaseAWSCommandParamsTest):
    def setUp(self):
        super(BaseCLIInputArgumentTest, self).setUp()
        self.files = FileCreator()

    def tearDown(self):
        super(BaseCLIInputArgumentTest, self).tearDown()
        self.files.remove_all()


class TestCLIInputJSON(BaseCLIInputArgumentTest):
    def setUp(self):
        super(TestCLIInputJSON, self).setUp()
        self.input_json = '{"Bucket": "bucket", "Key": "key"}'
        self.input_file = self.files.create_file('foo.json', self.input_json)

    def test_cli_input_json_no_exta_args(self):
        # Run a head command using the input json
        cmdline = (
            's3api head-object --cli-input-json file://%s'
        ) % self.input_file
        self.assert_params_for_cmd(
            cmdline, params={'Bucket': 'bucket', 'Key': 'key'}
        )

    def test_cli_input_json_can_override_param(self):
        cmdline = (
            's3api head-object --key bar --cli-input-json file://%s'
        ) % self.input_file
        self.assert_params_for_cmd(cmdline, {'Bucket': 'bucket', 'Key': 'bar'})

    def test_cli_input_json_not_from_file(self):
        # Check that the input json can be used without having to use a file.
        cmdline = (
            's3api head-object --cli-input-json '
            '{"Bucket":"bucket","Key":"key"}'
        )
        self.assert_params_for_cmd(
            cmdline, params={'Bucket': 'bucket', 'Key': 'key'}
        )

    def test_cli_input_json_missing_required(self):
        # Check that the operation properly throws an error if the json is
        # missing any required arguments and the argument is not on the
        # command line.
        cmdline = 's3api head-object --cli-input-json {"Key":"foo"}'
        self.assert_params_for_cmd(
            cmdline, expected_rc=252, stderr_contains='Missing'
        )

    def test_cli_input_json_has_extra_unknown_args(self):
        # Check that the operation properly throws an error if the json
        # has an extra argument that is not defined by the model.
        cmdline = (
            's3api head-object --cli-input-json '
            '{"Bucket":"bucket","Key":"key","Foo":"bar"}'
        )
        self.assert_params_for_cmd(
            cmdline, expected_rc=252, stderr_contains='Unknown'
        )


class TestCLIInputYAML(BaseCLIInputArgumentTest):
    def test_input_yaml(self):
        command = [
            's3api',
            'list-objects-v2',
            '--cli-input-yaml',
            'Bucket: test-bucket\nEncodingType: url',
        ]
        self.assert_params_for_cmd(
            command, {'Bucket': 'test-bucket', 'EncodingType': 'url'}
        )

    def test_unknown_params_in_input(self):
        command = [
            's3api',
            'list-objects-v2',
            '--cli-input-yaml',
            'Bucket: test-bucket\nFoo: bar',
        ]
        self.run_cmd(command, expected_rc=252)

    def test_required_params_missing(self):
        command = [
            's3api',
            'list-objects-v2',
            '--cli-input-yaml',
            'EncodingType: url',
        ]
        self.run_cmd(command, expected_rc=252)

    def test_input_yaml_file(self):
        filename = self.files.create_file(
            'input.yml', 'Bucket: test-bucket\nEncodingType: url'
        )
        command = [
            's3api',
            'list-objects-v2',
            '--cli-input-yaml',
            'file://%s' % filename,
        ]
        self.assert_params_for_cmd(
            command, {'Bucket': 'test-bucket', 'EncodingType': 'url'}
        )

    def test_input_yaml_bytes(self):
        expected_args = {
            'Bucket': 'mybucket',
            'Key': 'mykey',
            'Body': b'foo',
        }
        command = [
            's3api',
            'put-object',
            '--cli-input-yaml',
            dump_yaml_to_str(self.yaml, expected_args),
        ]
        self.run_cmd(command)
        self.assertEqual(self.last_kwargs['Body'].getvalue(), b'foo')

    def test_input_yaml_ignores_comments(self):
        command = [
            's3api',
            'list-objects-v2',
            '--cli-input-yaml',
            'Bucket: test-bucket # Some comment\nEncodingType: url',
        ]
        self.assert_params_for_cmd(
            command, {'Bucket': 'test-bucket', 'EncodingType': 'url'}
        )

    def test_errors_when_both_yaml_and_json_provided(self):
        command = [
            's3api',
            'list-objects-v2',
            '--cli-input-yaml',
            'Bucket: test-bucket # Some comment\nEncodingType: url',
            '--cli-input-json',
            '{"Bucket":"bucket","Key":"key"}',
        ]
        self.assert_params_for_cmd(
            command,
            expected_rc=252,
            stderr_contains='Only one --cli-input- parameter may be specified.',
        )


class TestBinaryInput(BaseCLIInputArgumentTest):
    def test_converts_base64_to_binary_in_base64_mode(self):
        command = [
            'kms',
            'encrypt',
            '--cli-binary-format',
            'base64',
            '--key-id',
            'test',
            '--plaintext',
            'Zm9v',
        ]
        params = {
            'KeyId': 'test',
            'Plaintext': b'foo',
        }
        self.assert_params_for_cmd(command, params)

    def test_preserved_input_value_in_raw_in_base64_out_mode(self):
        command = [
            'kms',
            'encrypt',
            '--cli-binary-format',
            'raw-in-base64-out',
            '--key-id',
            'test',
            '--plaintext',
            'Zm9v',
        ]
        params = {
            'KeyId': 'test',
            'Plaintext': 'Zm9v',
        }
        self.assert_params_for_cmd(command, params)



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import botocore.session
from botocore.compat import json

from awscli.customizations.dlm.constants import (
    LIFECYCLE_DEFAULT_MANAGED_POLICY_NAME,
    LIFECYCLE_DEFAULT_MANAGED_POLICY_NAME_AMI,
    LIFECYCLE_DEFAULT_ROLE_ASSUME_POLICY,
    LIFECYCLE_DEFAULT_ROLE_NAME,
    LIFECYCLE_DEFAULT_ROLE_NAME_AMI,
    RESOURCE_TYPE_IMAGE,
    RESOURCE_TYPE_SNAPSHOT,
)
from awscli.customizations.dlm.iam import IAM
from awscli.testutils import BaseAWSCommandParamsTest, mock, unittest


class TestCreateDefaultRole(BaseAWSCommandParamsTest):
    prefix = 'dlm create-default-role'
    LIFECYCLE_DEFAULT_MANAGED_POLICY_ARN = (
        "arn:aws:iam::aws:policy/service-role/%s"
        % (LIFECYCLE_DEFAULT_MANAGED_POLICY_NAME)
    )
    LIFECYCLE_DEFAULT_MANAGED_POLICY_AMI_ARN = (
        "arn:aws:iam::aws:policy/service-role/%s"
        % (LIFECYCLE_DEFAULT_MANAGED_POLICY_NAME_AMI)
    )

    # Call to attach policy to role
    def assert_attached_policy_to_role(
        self, expected_policy_arn, expected_role
    ):
        self.assertEqual(self.operations_called[2][0].name, 'AttachRolePolicy')
        self.assertEqual(
            self.operations_called[2][1]['PolicyArn'], expected_policy_arn
        )
        self.assertEqual(
            self.operations_called[2][1]['RoleName'], expected_role
        )

    # Call to create default role
    def assert_create_default_role(self, role, assume_policy):
        self.assertEqual(self.operations_called[1][0].name, 'CreateRole')
        self.assertEqual(self.operations_called[1][1]['RoleName'], role)
        self.assertEqual(
            self.operations_called[1][1]['AssumeRolePolicyDocument'],
            json.dumps(assume_policy),
        )

    # Use case: Default role exists
    # Expected results: No Operation performed for creation,
    # only call made for verifying existence of role
    # create-default-role is executed without any resource type parameter,
    # should default to snapshot
    def test_default_role_exists(self):
        cmdline = self.prefix

        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(len(self.operations_called), 1)

        # Call to check if default lifecycle role exists
        self.assertEqual(self.operations_called[0][0].name, 'GetRole')
        self.assertEqual(
            self.operations_called[0][1]['RoleName'],
            LIFECYCLE_DEFAULT_ROLE_NAME,
        )

    # Use case: Default role does not exist.
    # Managed Policy exists.
    # Expected results: Operations are performed by the client to verify
    # existence of policy, creation of role and then
    # attaching policy to role
    # create-default-role is executed without any resource type parameter,
    # should default to snapshot
    @mock.patch('awscli.customizations.dlm.' 'iam.IAM.check_if_role_exists')
    def test_default_role_not_exist(self, role_exists_patch):
        role_exists_patch.return_value = False

        self.run_cmd(self.prefix, expected_rc=0)
        self.assertEqual(len(self.operations_called), 5)

        # Call to check if managed policy exists.
        self.assertEqual(self.operations_called[0][0].name, 'GetPolicy')
        self.assertEqual(
            self.operations_called[0][1]['PolicyArn'],
            self.LIFECYCLE_DEFAULT_MANAGED_POLICY_ARN,
        )

        self.assert_create_default_role(
            LIFECYCLE_DEFAULT_ROLE_NAME, LIFECYCLE_DEFAULT_ROLE_ASSUME_POLICY
        )
        self.assert_attached_policy_to_role(
            self.LIFECYCLE_DEFAULT_MANAGED_POLICY_ARN,
            LIFECYCLE_DEFAULT_ROLE_NAME,
        )

        # Call to get policy's default version id
        self.assertEqual(self.operations_called[3][0].name, 'GetPolicy')
        self.assertEqual(
            self.operations_called[3][1]['PolicyArn'],
            self.LIFECYCLE_DEFAULT_MANAGED_POLICY_ARN,
        )

        # Call to get detailed policy to
        # construct result with policy permissions
        self.assertEqual(self.operations_called[4][0].name, 'GetPolicyVersion')
        self.assertEqual(
            self.operations_called[4][1]['PolicyArn'],
            self.LIFECYCLE_DEFAULT_MANAGED_POLICY_ARN,
        )

    # Use case: Default role exists
    # Expected results: No Operation performed for creation,
    # only call made for verifying existence of role
    # create-default-role is executed with resource type = snapshot
    def test_default_role_exists_snapshot(self):
        cmdline = self.prefix + " --resource-type=" + RESOURCE_TYPE_SNAPSHOT

        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(len(self.operations_called), 1)

        # Call to check if default lifecycle role exists
        self.assertEqual(self.operations_called[0][0].name, 'GetRole')
        self.assertEqual(
            self.operations_called[0][1]['RoleName'],
            LIFECYCLE_DEFAULT_ROLE_NAME,
        )

    # Use case: Default role does not exist.
    # Managed Policy exists.
    # Expected results: Operations are performed by the client to verify
    # existence of policy, creation of role and then
    # attaching policy to role
    # create-default-role is executed resource type = snapshot
    @mock.patch('awscli.customizations.dlm.' 'iam.IAM.check_if_role_exists')
    def test_default_role_not_exist_snapshot(self, role_exists_patch):
        role_exists_patch.return_value = False

        self.run_cmd(
            self.prefix + " --resource-type=%s" % (RESOURCE_TYPE_SNAPSHOT),
            expected_rc=0,
        )
        self.assertEqual(len(self.operations_called), 5)

        # Call to check if managed policy exists.
        self.assertEqual(self.operations_called[0][0].name, 'GetPolicy')
        self.assertEqual(
            self.operations_called[0][1]['PolicyArn'],
            self.LIFECYCLE_DEFAULT_MANAGED_POLICY_ARN,
        )

        self.assert_create_default_role(
            LIFECYCLE_DEFAULT_ROLE_NAME, LIFECYCLE_DEFAULT_ROLE_ASSUME_POLICY
        )
        self.assert_attached_policy_to_role(
            self.LIFECYCLE_DEFAULT_MANAGED_POLICY_ARN,
            LIFECYCLE_DEFAULT_ROLE_NAME,
        )

        # Call to get policy's default version id
        self.assertEqual(self.operations_called[3][0].name, 'GetPolicy')
        self.assertEqual(
            self.operations_called[3][1]['PolicyArn'],
            self.LIFECYCLE_DEFAULT_MANAGED_POLICY_ARN,
        )

        # Call to get detailed policy to
        # construct result with policy permissions
        self.assertEqual(self.operations_called[4][0].name, 'GetPolicyVersion')
        self.assertEqual(
            self.operations_called[4][1]['PolicyArn'],
            self.LIFECYCLE_DEFAULT_MANAGED_POLICY_ARN,
        )

    # Use case: Default role exists for AMI
    # Expected results: No Operation performed for creation,
    # only call made for verifying existence of role
    # create-default-role is executed with resource type = image
    def test_default_role_exists_ami(self):
        cmdline = self.prefix + " --resource-type=" + RESOURCE_TYPE_IMAGE

        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(len(self.operations_called), 1)

        # Call to check if default lifecycle role exists
        self.assertEqual(self.operations_called[0][0].name, 'GetRole')
        self.assertEqual(
            self.operations_called[0][1]['RoleName'],
            LIFECYCLE_DEFAULT_ROLE_NAME_AMI,
        )

    # Use case: Default role does not exist for AMI.
    # AMI Managed Policy exists.
    # Expected results: Operations are performed by the client to verify
    # existence of policy, creation of role and then
    # attaching policy to role
    # create-default-role is executed with resource type = image
    @mock.patch('awscli.customizations.dlm.' 'iam.IAM.check_if_role_exists')
    def test_default_role_not_exist_ami(self, role_exists_patch):
        role_exists_patch.return_value = False

        self.run_cmd(
            self.prefix + " --resource-type=%s" % (RESOURCE_TYPE_IMAGE),
            expected_rc=0,
        )
        self.assertEqual(len(self.operations_called), 5)

        # Call to check if managed policy exists.
        self.assertEqual(self.operations_called[0][0].name, 'GetPolicy')
        self.assertEqual(
            self.operations_called[0][1]['PolicyArn'],
            self.LIFECYCLE_DEFAULT_MANAGED_POLICY_AMI_ARN,
        )

        self.assert_create_default_role(
            LIFECYCLE_DEFAULT_ROLE_NAME_AMI,
            LIFECYCLE_DEFAULT_ROLE_ASSUME_POLICY,
        )
        self.assert_attached_policy_to_role(
            self.LIFECYCLE_DEFAULT_MANAGED_POLICY_AMI_ARN,
            LIFECYCLE_DEFAULT_ROLE_NAME_AMI,
        )

        # Call to get policy's default version id
        self.assertEqual(self.operations_called[3][0].name, 'GetPolicy')
        self.assertEqual(
            self.operations_called[3][1]['PolicyArn'],
            self.LIFECYCLE_DEFAULT_MANAGED_POLICY_AMI_ARN,
        )

        # Call to get detailed policy to
        # construct result with policy permissions
        self.assertEqual(self.operations_called[4][0].name, 'GetPolicyVersion')
        self.assertEqual(
            self.operations_called[4][1]['PolicyArn'],
            self.LIFECYCLE_DEFAULT_MANAGED_POLICY_AMI_ARN,
        )


class TestCreateDefaultRoleUnitTest(unittest.TestCase):
    def setUp(self):
        self.iam_client = mock.Mock()
        self.iam_client.exceptions.NoSuchEntityException = (
            botocore.session.get_session()
            .create_client('iam', region_name="us-east-1")
            .exceptions.NoSuchEntityException
        )
        self.iam = IAM(self.iam_client)

    def test_check_if_role_exists_raises_client_error(self):
        self.iam_client.get_role.side_effect = (
            self.iam_client.exceptions.NoSuchEntityException(
                error_response={'Error': {'Code': 'NoSuchEntityError'}},
                operation_name='GetRole',
            )
        )

        self.assertFalse(self.iam.check_if_role_exists('role'))

    def test_check_if_policy_exists_raises_client_error(self):
        self.iam_client.get_policy.side_effect = (
            self.iam_client.exceptions.NoSuchEntityException(
                error_response={'Error': {'Code': 'NoSuchEntityError'}},
                operation_name='GetPolicy',
            )
        )
        self.assertFalse(self.iam.check_if_policy_exists('policy'))


if __name__ == "__main__":
    unittest.main()



# Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from argparse import Namespace

from botocore import config

from awscli.customizations.ecs.deploy import ECSClient, ECSDeploy
from awscli.testutils import capture_output, mock, unittest


class TestECSClient(unittest.TestCase):
    def setUp(self):
        ecs_client = mock.Mock()
        self.session = mock.Mock()
        self.session.create_client.side_effect = ecs_client

        # set global args
        self.global_args = Namespace()
        self.global_args.region = 'us-east-1'
        self.global_args.endpoint_url = None
        self.global_args.verify_ssl = None

    def test_client_config(self):
        self.test_client = ECSClient(
            self.session, None, self.global_args, ECSDeploy.USER_AGENT_EXTRA
        )

        expected_user_agent_extra = 'md/customization#ecs-deploy'

        create_args = self.session.create_client.call_args
        self.assertEqual(create_args[0][0], 'ecs')
        self.assertEqual(
            create_args[1]['region_name'], self.global_args.region
        )
        self.assertEqual(
            create_args[1]['config'].user_agent_extra,
            expected_user_agent_extra,
        )



#!/usr/bin/env python
# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import awscli.clidriver
from awscli.testutils import BaseAWSCommandParamsTest


class TestGetQueueAttributes(BaseAWSCommandParamsTest):
    prefix = 'sqs get-queue-attributes'
    queue_url = 'https://queue.amazonaws.com/4444/testcli'

    def test_no_attr(self):
        cmdline = self.prefix + ' --queue-url %s' % self.queue_url
        result = {'QueueUrl': self.queue_url}
        self.assert_params_for_cmd(cmdline, result)

    def test_all(self):
        cmdline = self.prefix + ' --queue-url %s' % self.queue_url
        cmdline += ' --attribute-names All'
        result = {'QueueUrl': self.queue_url, 'AttributeNames': ['All']}
        self.assert_params_for_cmd(cmdline, result)

    def test_one(self):
        cmdline = self.prefix + ' --queue-url %s' % self.queue_url
        cmdline += ' --attribute-names VisibilityTimeout'
        result = {
            'QueueUrl': self.queue_url,
            'AttributeNames': ['VisibilityTimeout'],
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_two(self):
        cmdline = self.prefix + ' --queue-url %s' % self.queue_url
        cmdline += ' --attribute-names VisibilityTimeout QueueArn'
        result = {
            'QueueUrl': self.queue_url,
            'AttributeNames': ['VisibilityTimeout', 'QueueArn'],
        }
        self.assert_params_for_cmd(cmdline, result)


if __name__ == "__main__":
    unittest.main()



# Copyright 2021 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import awscrt.s3
import pytest
import s3transfer.crt
from awscrt.s3 import S3FileIoOptions, S3RequestTlsMode
from botocore.config import Config
from botocore.credentials import Credentials
from botocore.httpsession import DEFAULT_CA_BUNDLE
from botocore.session import Session
from s3transfer.crt import CRTTransferManager
from s3transfer.manager import TransferManager

from awscli.customizations.s3.factory import (
    ClientFactory,
    TransferManagerFactory,
)
from awscli.customizations.s3.transferconfig import RuntimeConfig
from awscli.testutils import FileCreator, mock, unittest


@pytest.fixture
def mock_crt_is_optimized_for_system():
    with mock.patch('awscrt.s3.is_optimized_for_system') as mock_is_optimized:
        mock_is_optimized.return_value = False
        yield mock_is_optimized


@pytest.fixture
def mock_crt_process_lock(monkeypatch):
    # The process lock is cached at the module layer whenever the
    # cross process lock is successfully acquired. This patch ensures that
    # test cases will start off with no previously cached process lock and
    # if a cross process is instantiated/acquired it will be the mock that
    # can be used for controlling lock behavior.
    monkeypatch.setattr('s3transfer.crt.CRT_S3_PROCESS_LOCK', None)
    with mock.patch('awscrt.s3.CrossProcessLock', spec=True) as mock_lock:
        yield mock_lock


@pytest.fixture
def mock_crt_s3_client():
    with mock.patch('s3transfer.crt.S3Client', spec=True) as mock_client:
        yield mock_client


@pytest.fixture
def transfer_manager_factory():
    session = mock.Mock(Session)
    session.get_config_variable.return_value = None
    session.get_default_client_config.return_value = None
    session.get_scoped_config.return_value = {}
    return TransferManagerFactory(session)


@pytest.fixture
def s3_params():
    return {
        'region': 'us-west-2',
        'endpoint_url': None,
        'verify_ssl': None,
    }


def test_crt_get_optimized_platforms_match_expected_platforms():
    expected_platforms = {
        'p4d.24xlarge',
        'p4de.24xlarge',
        'p5.48xlarge',
        'trn1n.32xlarge',
        'trn1.32xlarge',
    }
    actual_platforms = set(awscrt.s3.get_optimized_platforms())
    assert expected_platforms == actual_platforms, (
        'Expected set of CRT optimized platforms does not match result from '
        'CRT. The result from CRT determines which platforms the CLI will '
        'automatically use the S3 CRT client for. If the change in optimized '
        'platforms is expected/appropriate, update the expected_platforms '
        'set in this test and the list of CRT optimized platforms in the '
        'documentation located at: awscli/topics/s3-config.rst'
    )


class TestClientFactory(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock(Session)
        self.factory = ClientFactory(self.session)

    def test_create_client(self):
        params = {
            'region': 'us-west-2',
            'endpoint_url': 'https://myendpoint',
            'verify_ssl': True,
        }
        self.factory.create_client(params=params)
        self.session.create_client.assert_called_with(
            's3',
            region_name='us-west-2',
            endpoint_url='https://myendpoint',
            verify=True,
        )

    def test_create_client_sets_sigv4_for_sse_kms(self):
        params = {
            'region': 'us-west-2',
            'endpoint_url': None,
            'verify_ssl': None,
            'sse': 'aws:kms',
        }
        self.factory.create_client(params)
        self.assertEqual(
            self.session.create_client.call_args[1][
                'config'
            ].signature_version,
            's3v4',
        )

    def test_create_client_with_no_source_region(self):
        params = {
            'region': 'us-west-2',
            'endpoint_url': 'https://myendpoint',
            'verify_ssl': True,
            'source_region': None,
        }
        self.factory.create_client(params, is_source_client=True)
        self.session.create_client.assert_called_with(
            's3',
            region_name='us-west-2',
            endpoint_url='https://myendpoint',
            verify=True,
        )

    def test_create_client_respects_source_region_for_copies(self):
        params = {
            'region': 'us-west-2',
            'endpoint_url': 'https://myendpoint',
            'verify_ssl': True,
            'source_region': 'us-west-1',
            'paths_type': 's3s3',
        }
        self.factory.create_client(params, is_source_client=True)
        self.session.create_client.assert_called_with(
            's3', region_name='us-west-1', endpoint_url=None, verify=True
        )


class TestTransferManagerFactory(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock(Session)
        self.session.get_config_variable.return_value = None
        self.session.get_default_client_config.return_value = None
        self.session.get_scoped_config.return_value = {}
        self.factory = TransferManagerFactory(self.session)
        self.params = {
            'region': 'us-west-2',
            'endpoint_url': None,
            'verify_ssl': None,
        }
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='classic'
        )
        self.files = FileCreator()

    def tearDown(self):
        self.files.remove_all()

    def get_runtime_config(self, **kwargs):
        return RuntimeConfig().build_config(**kwargs)

    def assert_tls_enabled_for_crt_client(self, mock_crt_client):
        self.assertEqual(
            mock_crt_client.call_args[1]['tls_mode'], S3RequestTlsMode.ENABLED
        )

    def assert_tls_disabled_for_crt_client(self, mock_crt_client):
        self.assertEqual(
            mock_crt_client.call_args[1]['tls_mode'], S3RequestTlsMode.DISABLED
        )

    def assert_uses_client_tls_context_options(
        self, mock_crt_client, mock_client_tls_context_options
    ):
        mock_connection_options = mock_client_tls_context_options.return_value.new_connection_options.return_value
        self.assertIs(
            mock_crt_client.call_args[1]['tls_connection_options'],
            mock_connection_options,
        )

    def assert_is_classic_manager(self, manager):
        self.assertIsInstance(manager, TransferManager)

    def assert_is_crt_manager(self, manager):
        self.assertIsInstance(manager, CRTTransferManager)

    def assert_expected_throughput_target_gbps(
        self, mock_crt_client, expected_throughput_target_gbps
    ):
        self.assertEqual(
            mock_crt_client.call_args[1]['throughput_target_gbps'],
            expected_throughput_target_gbps,
        )

    def test_create_transfer_manager_classic(self):
        transfer_client = mock.Mock()
        self.session.create_client.return_value = transfer_client
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_classic_manager(transfer_manager)
        self.session.create_client.assert_called_with(
            's3',
            region_name='us-west-2',
            endpoint_url=None,
            verify=None,
        )
        self.assertIs(transfer_manager.client, transfer_client)

    def test_proxies_transfer_config_to_default_transfer_manager(self):
        MB = 1024**2
        self.runtime_config = self.get_runtime_config(
            multipart_chunksize=5 * MB,
            multipart_threshold=20 * MB,
            max_concurrent_requests=20,
            max_queue_size=5000,
            max_bandwidth=10 * MB,
            io_chunksize=1 * MB,
        )
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assertEqual(transfer_manager.config.multipart_chunksize, 5 * MB)
        self.assertEqual(transfer_manager.config.multipart_threshold, 20 * MB)
        self.assertEqual(transfer_manager.config.max_request_concurrency, 20)
        self.assertEqual(transfer_manager.config.max_request_queue_size, 5000)
        self.assertEqual(transfer_manager.config.max_bandwidth, 10 * MB)
        self.assertEqual(transfer_manager.config.io_chunksize, 1 * MB)
        # These configurations are hardcoded and not configurable but
        # we just want to make sure they are being set by the factory.
        self.assertEqual(
            transfer_manager.config.max_in_memory_upload_chunks, 6
        )
        self.assertEqual(
            transfer_manager.config.max_in_memory_upload_chunks, 6
        )

    def test_can_provide_botocore_client_to_classic_manager(self):
        transfer_client = mock.Mock()
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config, botocore_client=transfer_client
        )
        self.assert_is_classic_manager(transfer_manager)
        self.session.create_client.assert_not_called()
        self.assertIs(transfer_manager.client, transfer_client)

    @mock.patch('s3transfer.crt.S3Client')
    def test_uses_region_parameter_for_crt_manager(self, mock_crt_client):
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt'
        )
        self.params['region'] = 'param-region'
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        self.assertEqual(
            mock_crt_client.call_args[1]['region'], 'param-region'
        )
        self.assertEqual(
            self.session.create_client.call_args[1]['region_name'],
            'param-region',
        )

    @mock.patch('s3transfer.crt.S3Client')
    def test_falls_back_to_session_region_for_crt_manager(
        self, mock_crt_client
    ):
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt'
        )
        params = {'verify_ssl': DEFAULT_CA_BUNDLE}
        self.session.get_config_variable.return_value = 'config-region'
        transfer_manager = self.factory.create_transfer_manager(
            params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        self.assertEqual(
            mock_crt_client.call_args[1]['region'], 'config-region'
        )
        self.assertEqual(
            self.session.create_client.call_args[1]['region_name'],
            'config-region',
        )

    @mock.patch('s3transfer.crt.S3Client')
    def test_uses_tls_by_default_for_crt_manager(self, mock_crt_client):
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt'
        )
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        self.assert_tls_enabled_for_crt_client(mock_crt_client)

    @mock.patch('s3transfer.crt.S3Client')
    def test_uses_endpoint_url_parameter_for_crt_manager(
        self, mock_crt_client
    ):
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt'
        )
        self.params['endpoint_url'] = 'https://my.endpoint.com'
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        self.assertEqual(
            self.session.create_client.call_args[1]['endpoint_url'],
            'https://my.endpoint.com',
        )
        self.assert_tls_enabled_for_crt_client(mock_crt_client)

    @mock.patch('s3transfer.crt.S3Client')
    def test_can_disable_tls_using_endpoint_scheme_for_crt_manager(
        self, mock_crt_client
    ):
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt'
        )
        self.params['endpoint_url'] = 'http://my.endpoint.com'
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        self.assertEqual(
            self.session.create_client.call_args[1]['endpoint_url'],
            'http://my.endpoint.com',
        )
        self.assert_tls_disabled_for_crt_client(mock_crt_client)

    @mock.patch('s3transfer.crt.S3Client')
    def test_uses_botocore_credentials_for_crt_manager(self, mock_crt_client):
        credentials = Credentials('access_key', 'secret_key', 'token')
        self.session.get_credentials.return_value = credentials
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt'
        )
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        self.session.get_credentials.assert_called_with()
        crt_credential_provider = mock_crt_client.call_args[1][
            'credential_provider'
        ]
        self.assertIsNotNone(crt_credential_provider)

        # Ensure the credentials returned by the CRT credential provider
        # match the session's credentials
        future = crt_credential_provider.get_credentials()
        crt_credentials = future.result()
        assert crt_credentials.access_key_id == 'access_key'
        assert crt_credentials.secret_access_key == 'secret_key'
        assert crt_credentials.session_token == 'token'

    @mock.patch('s3transfer.crt.S3Client')
    def test_disable_botocore_credentials_for_crt_manager(
        self, mock_crt_client
    ):
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt'
        )
        self.params['sign_request'] = False
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        self.session.get_credentials.assert_not_called()
        self.assertIsNone(mock_crt_client.call_args[1]['credential_provider'])

    @mock.patch('s3transfer.crt.S3Client')
    @mock.patch('s3transfer.crt.ClientTlsContext')
    def test_use_verify_ssl_parameter_for_crt_manager(
        self, mock_client_tls_context_options, mock_crt_client
    ):
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt'
        )
        fake_ca_contents = b"fake ca content"
        fake_ca_bundle = self.files.create_file(
            "fake_ca", fake_ca_contents, mode='wb'
        )
        self.params['verify_ssl'] = fake_ca_bundle
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        tls_context_options = mock_client_tls_context_options.call_args[0][0]
        self.assertEqual(tls_context_options.ca_buffer, fake_ca_contents)
        self.assert_uses_client_tls_context_options(
            mock_crt_client, mock_client_tls_context_options
        )

    @mock.patch('s3transfer.crt.S3Client')
    @mock.patch('s3transfer.crt.ClientTlsContext')
    def test_use_ca_bundle_from_session_for_crt_manager(
        self, mock_client_tls_context_options, mock_crt_client
    ):
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt'
        )
        fake_ca_contents = b"fake ca content"
        fake_ca_bundle = self.files.create_file(
            "fake_ca", fake_ca_contents, mode='wb'
        )
        self.session.get_config_variable.return_value = fake_ca_bundle
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        tls_context_options = mock_client_tls_context_options.call_args[0][0]
        self.assertEqual(tls_context_options.ca_buffer, fake_ca_contents)
        self.assert_uses_client_tls_context_options(
            mock_crt_client, mock_client_tls_context_options
        )

    @mock.patch('s3transfer.crt.S3Client')
    @mock.patch('s3transfer.crt.ClientTlsContext')
    def test_use_verify_ssl_parameter_none_for_crt_manager(
        self, mock_client_tls_context_options, mock_crt_client
    ):
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt'
        )
        self.params['verify_ssl'] = None
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        tls_context_options = mock_client_tls_context_options.call_args[0][0]
        with open(DEFAULT_CA_BUNDLE, mode='rb') as fh:
            contents = fh.read()
            self.assertEqual(tls_context_options.ca_buffer, contents)
        self.assert_uses_client_tls_context_options(
            mock_crt_client, mock_client_tls_context_options
        )

    @mock.patch('s3transfer.crt.S3Client')
    @mock.patch('s3transfer.crt.ClientTlsContext')
    def test_use_verify_ssl_parameter_false_for_crt_manager(
        self, mock_client_tls_context_options, mock_crt_client
    ):
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt'
        )
        self.params['verify_ssl'] = False
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        tls_context_options = mock_client_tls_context_options.call_args[0][0]
        self.assertFalse(tls_context_options.verify_peer)
        self.assert_uses_client_tls_context_options(
            mock_crt_client, mock_client_tls_context_options
        )

    @mock.patch('s3transfer.crt.S3Client')
    def test_target_bandwidth_configure_for_crt_manager(self, mock_crt_client):
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt', target_bandwidth=1_000_000_000
        )
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        self.assert_expected_throughput_target_gbps(mock_crt_client, 8)

    @mock.patch('s3transfer.crt.S3Client')
    def test_fio_options_configure_for_crt_manager(self, mock_crt_client):
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt',
            should_stream=True,
            disk_throughput=1000**3,
            direct_io=True,
        )
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        expected_fio_options = S3FileIoOptions(
            should_stream=True,
            disk_throughput_gbps=8.0,
            direct_io=True,
        )
        self.assert_is_crt_manager(transfer_manager)
        self.assertEqual(
            mock_crt_client.call_args[1]['fio_options'], expected_fio_options
        )

    @mock.patch('s3transfer.crt.get_recommended_throughput_target_gbps')
    @mock.patch('s3transfer.crt.S3Client')
    def test_target_bandwidth_uses_crt_recommended_throughput(
        self, mock_crt_client, mock_get_target_gbps
    ):
        mock_get_target_gbps.return_value = 100
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt',
        )
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        self.assert_expected_throughput_target_gbps(mock_crt_client, 100)

    @mock.patch('s3transfer.crt.get_recommended_throughput_target_gbps')
    @mock.patch('s3transfer.crt.S3Client')
    def test_crt_recommended_target_throughput_default(
        self, mock_crt_client, mock_get_target_gbps
    ):
        mock_get_target_gbps.return_value = None
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt',
        )
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        # Default when CRT is unable to recommend a throughput
        # should be 10 gbps
        self.assert_expected_throughput_target_gbps(mock_crt_client, 10)

    @mock.patch('s3transfer.crt.S3Client')
    def test_multipart_chunksize_configure_for_crt_manager(
        self, mock_crt_client
    ):
        part_size = 16 * (1024**2)
        self.session.get_scoped_config.return_value = {
            's3': {
                'multipart_chunksize': part_size,
            }
        }
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt', multipart_chunksize=part_size
        )
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        self.assertEqual(mock_crt_client.call_args[1]['part_size'], part_size)

    @mock.patch('s3transfer.crt.S3Client')
    def test_default_part_size_for_crt_manager(self, mock_crt_client):
        part_size = 16 * (1024**2)
        # Explicitly showing that the user has not configured
        # `multipart_chunksize`.
        self.session.get_scoped_config.return_value = {'s3': {}}
        self.runtime_config = self.get_runtime_config(
            preferred_transfer_client='crt', multipart_chunksize=part_size
        )
        transfer_manager = self.factory.create_transfer_manager(
            self.params, self.runtime_config
        )
        self.assert_is_crt_manager(transfer_manager)
        # When `multipart_chunksize` isn't explicitly provided, configure
        # `part_size` to `None`.
        self.assertEqual(mock_crt_client.call_args[1]['part_size'], None)


@pytest.mark.parametrize(
    'preferred_transfer_client,extra_params,'
    'crt_is_optimized_for_system,crt_running_in_other_process,'
    'expected_transfer_manager_cls',
    [
        (None, {}, False, False, TransferManager),
        ('auto', {}, False, False, TransferManager),
        ('classic', {}, False, False, TransferManager),
        ('crt', {}, False, False, CRTTransferManager),
        # "default" is a supported alias for "classic"
        ('default', {}, False, False, TransferManager),
        # Cases when CRT is optimized for system
        (None, {}, True, False, CRTTransferManager),
        ('auto', {}, True, False, CRTTransferManager),
        ('classic', {}, True, False, TransferManager),
        ('crt', {}, True, False, CRTTransferManager),
        # Cases when another AWS CLI process is running CRT
        (None, {}, True, True, TransferManager),
        ('auto', {}, True, True, TransferManager),
        ('classic', {}, True, True, TransferManager),
        ('crt', {}, True, True, CRTTransferManager),
        # S3 copies always default to classic transfer manager
        (None, {'paths_type': 's3s3'}, True, False, TransferManager),
        ('auto', {'paths_type': 's3s3'}, True, False, TransferManager),
        ('classic', {'paths_type': 's3s3'}, True, False, TransferManager),
        ('crt', {'paths_type': 's3s3'}, True, False, TransferManager),
        # Streaming operations use requested transfer client
        (None, {'is_stream': True}, False, False, TransferManager),
        ('auto', {'is_stream': True}, False, False, TransferManager),
        ('classic', {'is_stream': True}, False, False, TransferManager),
        ('crt', {'is_stream': True}, False, False, CRTTransferManager),
    ],
)
def test_transfer_manager_cls_resolution(
    preferred_transfer_client,
    extra_params,
    crt_is_optimized_for_system,
    crt_running_in_other_process,
    expected_transfer_manager_cls,
    transfer_manager_factory,
    s3_params,
    mock_crt_is_optimized_for_system,
    mock_crt_process_lock,
    mock_crt_s3_client,
):
    s3_params.update(extra_params)
    mock_crt_is_optimized_for_system.return_value = crt_is_optimized_for_system
    if crt_running_in_other_process:
        mock_crt_process_lock.return_value.acquire.side_effect = RuntimeError

    transfer_manager = _create_transfer_manager_from_factory(
        transfer_manager_factory, s3_params, preferred_transfer_client
    )
    assert isinstance(transfer_manager, expected_transfer_manager_cls)


@pytest.mark.parametrize(
    'preferred_transfer_client,crt_is_optimized_for_system',
    [
        ('auto', True),
        ('crt', False),
        ('crt', True),
    ],
)
def test_factory_always_acquires_crt_transfer_lock_for_crt_manager(
    preferred_transfer_client,
    crt_is_optimized_for_system,
    transfer_manager_factory,
    s3_params,
    mock_crt_is_optimized_for_system,
    mock_crt_process_lock,
    mock_crt_s3_client,
):
    mock_crt_is_optimized_for_system.return_value = crt_is_optimized_for_system
    transfer_manager = _create_transfer_manager_from_factory(
        transfer_manager_factory, s3_params, preferred_transfer_client
    )
    assert isinstance(transfer_manager, CRTTransferManager)
    assert s3transfer.crt.CRT_S3_PROCESS_LOCK
    mock_crt_process_lock.assert_called_once_with('aws-cli')
    mock_crt_process_lock.return_value.acquire.assert_called_once_with()


@pytest.mark.parametrize(
    'preferred_transfer_client,crt_is_optimized_for_system',
    [
        ('auto', False),
        ('classic', False),
        ('classic', True),
    ],
)
def test_factory_never_acquires_crt_transfer_lock_for_classic_manager(
    preferred_transfer_client,
    crt_is_optimized_for_system,
    transfer_manager_factory,
    s3_params,
    mock_crt_is_optimized_for_system,
    mock_crt_process_lock,
    mock_crt_s3_client,
):
    mock_crt_is_optimized_for_system.return_value = crt_is_optimized_for_system
    transfer_manager = _create_transfer_manager_from_factory(
        transfer_manager_factory, s3_params, preferred_transfer_client
    )
    assert isinstance(transfer_manager, TransferManager)
    assert s3transfer.crt.CRT_S3_PROCESS_LOCK is None
    mock_crt_process_lock.assert_not_called()
    mock_crt_process_lock.return_value.acquire.assert_not_called()


def _create_transfer_manager_from_factory(
    transfer_manager_factory, params, preferred_transfer_client=None
):
    runtime_config_kwargs = {}
    if preferred_transfer_client is not None:
        runtime_config_kwargs['preferred_transfer_client'] = (
            preferred_transfer_client
        )
    runtime_config = RuntimeConfig().build_config(**runtime_config_kwargs)
    return transfer_manager_factory.create_transfer_manager(
        params, runtime_config
    )



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.clidriver import AWSCLIEntryPoint
from awscli.customizations.configure.configure import ConfigureCommand
from awscli.testutils import (
    BaseAWSCommandParamsTest,
    FileCreator,
    create_clidriver,
    mock,
    unittest,
)


class TestConfigureCommand(BaseAWSCommandParamsTest):
    def setUp(self):
        super().setUp()
        self.files = FileCreator()
        self.config_filename = self.files.full_path("configure")
        self.environ["AWS_CONFIG_FILE"] = self.config_filename
        self.environ["AWS_SHARED_CREDENTIALS_FILE"] = "asdf-does-not-exist"

    def tearDown(self):
        super().tearDown()
        self.files.remove_all()

    def set_config_file_contents(self, contents):
        self.files.create_file(self.config_filename, contents)
        # Reset the session to pick up the new config file.
        self.driver = create_clidriver()
        self.entry_point = AWSCLIEntryPoint(self.driver)

    def get_config_file_contents(self):
        with open(self.config_filename) as f:
            return f.read()

    def test_list_command(self):
        self.set_config_file_contents(
            "\n"
            "[default]\n"
            "aws_access_key_id=12345\n"
            "aws_secret_access_key=12345\n"
            "region=us-west-2\n"
        )
        self.environ.pop("AWS_DEFAULT_REGION", None)
        self.environ.pop("AWS_ACCESS_KEY_ID", None)
        self.environ.pop("AWS_SECRET_ACCESS_KEY", None)
        stdout, _, _ = self.run_cmd("configure list")
        self.assertRegex(stdout, r"access_key.+config-file")
        self.assertRegex(stdout, r"secret_key.+config-file")
        self.assertRegex(stdout, r"region\s+:\sus-west-2\s+:\sconfig-file")

    def test_get_command(self):
        self.set_config_file_contents(
            "\n"
            "[default]\n"
            "aws_access_key_id=access_key\n"
            "aws_secret_access_key=secret_key\n"
            "region=us-west-2\n"
        )
        stdout, _, _ = self.run_cmd("configure get aws_access_key_id")
        self.assertEqual(stdout.strip(), "access_key")

    def test_get_command_with_profile_set(self):
        self.set_config_file_contents(
            "\n"
            "[default]\n"
            "aws_access_key_id=default_access_key\n"
            "\n"
            "[profile testing]\n"
            "aws_access_key_id=testing_access_key\n"
        )
        stdout, _, _ = self.run_cmd(
            "configure get aws_access_key_id --profile testing",
        )
        self.assertEqual(stdout.strip(), "testing_access_key")

    def test_get_command_with_nonexisting_profile(self):
        self.set_config_file_contents(
            "\n[default]\naws_access_key_id=default_access_key\n\n"
        )
        _, stderr, _ = self.run_cmd(
            "configure get --profile doesntexist sso_region",
            expected_rc=255,
        )
        self.assertIn("could not be found", stderr)

    def test_get_with_fq_name(self):
        # test get configs with fully qualified name.
        self.set_config_file_contents(
            "\n"
            "[default]\n"
            "aws_access_key_id=default_access_key\n"
            "\n"
            "[profile testing]\n"
            "aws_access_key_id=testing_access_key\n"
        )
        stdout, _, _ = self.run_cmd(
            "configure get default.aws_access_key_id --profile testing",
        )
        self.assertEqual(stdout.strip(), "default_access_key")

    def test_get_with_fq_profile_name(self):
        self.set_config_file_contents(
            "\n"
            "[default]\n"
            "aws_access_key_id=default_access_key\n"
            "\n"
            "[profile testing]\n"
            "aws_access_key_id=testing_access_key\n"
        )
        stdout, _, _ = self.run_cmd(
            "configure get profile.testing.aws_access_key_id "
            "--profile default",
        )
        self.assertEqual(stdout.strip(), "testing_access_key")

    def test_get_fq_with_quoted_profile_name(self):
        self.set_config_file_contents(
            "\n"
            "[default]\n"
            "aws_access_key_id=default_access_key\n"
            "\n"
            '[profile "testing"]\n'
            "aws_access_key_id=testing_access_key\n"
        )
        stdout, _, _ = self.run_cmd(
            "configure get profile.testing.aws_access_key_id "
            "--profile default",
        )
        self.assertEqual(stdout.strip(), "testing_access_key")

    def test_get_command_with_subsection(self):
        self.set_config_file_contents(
            "\n"
            "[default]\n"
            "aws_access_key_id=default_access_key\n"
            "\n"
            "[sso-session my-sso-session]\n"
            "sso_region = us-west-2\n"
        )
        stdout, _, _ = self.run_cmd(
            "configure get --sso-session my-sso-session sso_region",
        )
        self.assertEqual(stdout.strip(), "us-west-2")

    def test_get_command_with_subsection_nested_property(self):
        self.set_config_file_contents(
            "\n"
            "[default]\n"
            "aws_access_key_id=default_access_key\n"
            "\n"
            "[services my-services]\n"
            "ec2 = \n"
            "    endpoint_url = http://localhost:4567/"
        )
        stdout, _, _ = self.run_cmd(
            "configure get --services my-services ec2.endpoint_url",
        )
        self.assertEqual(stdout.strip(), "http://localhost:4567/")

    def test_get_command_with_nonexisting_subsection(self):
        self.set_config_file_contents(
            "\n[default]\naws_access_key_id=default_access_key\n\n"
        )
        _, stderr, _ = self.run_cmd(
            "configure get --sso-session my-sso-session sso_region",
            expected_rc=255,
        )
        self.assertIn("could not be found", stderr)

    def test_get_command_with_subsection_set_nonexisting_subsection(self):
        self.set_config_file_contents(
            "\n"
            "[default]\n"
            "aws_access_key_id=default_access_key\n"
            "\n"
            "[sso-session my-sso-session]\n"
            "sso_region = us-west-2\n"
        )
        _, stderr, _ = self.run_cmd(
            "configure get --services my-services ec2.endpoint_url",
            expected_rc=255,
        )
        self.assertIn("could not be found", stderr)

    def test_set_with_config_file_no_exist(self):
        self.run_cmd("configure set region us-west-1")
        self.assertEqual(
            "[default]\nregion = us-west-1\n",
            self.get_config_file_contents(),
        )

    def test_set_with_a_url(self):
        self.run_cmd(
            "configure set endpoint http://www.example.com",
        )
        self.assertEqual(
            "[default]\nendpoint = http://www.example.com\n",
            self.get_config_file_contents(),
        )

    def test_set_with_empty_config_file(self):
        with open(self.config_filename, "w"):
            pass

        self.run_cmd("configure set region us-west-1")
        self.assertEqual(
            "[default]\nregion = us-west-1\n",
            self.get_config_file_contents(),
        )

    def test_set_with_updating_value(self):
        self.set_config_file_contents("[default]\nregion = us-west-2\n")
        self.run_cmd("configure set region us-west-1")
        self.assertEqual(
            "[default]\nregion = us-west-1\n",
            self.get_config_file_contents(),
        )

    def test_set_with_profile_spaces(self):
        self.run_cmd(
            [
                "configure",
                "set",
                "region",
                "us-west-1",
                "--profile",
                "test with spaces",
            ]
        )
        self.assertEqual(
            "[profile 'test with spaces']\nregion = us-west-1\n",
            self.get_config_file_contents(),
        )

    def test_set_with_profile_unknown_nested_key(self):
        self.run_cmd(
            [
                "configure",
                "set",
                "un.known",
                "us-west-1",
                "--profile",
                "space test",
            ]
        )
        self.assertEqual(
            "[profile 'space test']\nun =\n    known = us-west-1\n",
            self.get_config_file_contents(),
        )

    def test_set_with_profile_spaces_scoped(self):
        self.run_cmd(
            [
                "configure",
                "set",
                "profile.test with spaces.region",
                "us-west-1",
            ]
        )
        self.assertEqual(
            "[profile 'test with spaces']\nregion = us-west-1\n",
            self.get_config_file_contents(),
        )

    def test_set_with_profile(self):
        self.run_cmd(
            "configure set region us-west-1 --profile testing",
        )
        self.assertEqual(
            "[profile testing]\nregion = us-west-1\n",
            self.get_config_file_contents(),
        )

    def test_set_with_fq_double_dot(self):
        self.run_cmd(
            "configure set profile.testing.region us-west-2",
        )
        self.assertEqual(
            "[profile testing]\nregion = us-west-2\n",
            self.get_config_file_contents(),
        )

    def test_set_with_triple_nesting(self):
        self.run_cmd(
            "configure set default.s3.signature_version s3v4",
        )
        self.assertEqual(
            "[default]\ns3 =\n    signature_version = s3v4\n",
            self.get_config_file_contents(),
        )

    def test_set_with_existing_config(self):
        self.set_config_file_contents(
            "[default]\n"
            "region = us-west-2\n"
            "ec2 =\n"
            "    signature_version = v4\n"
        )
        self.run_cmd(
            "configure set default.s3.signature_version s3v4",
        )
        self.assertEqual(
            "[default]\n"
            "region = us-west-2\n"
            "ec2 =\n"
            "    signature_version = v4\n"
            "s3 =\n"
            "    signature_version = s3v4\n",
            self.get_config_file_contents(),
        )

    def test_set_with_new_profile(self):
        self.set_config_file_contents(
            "[default]\ns3 =\n    signature_version = s3v4\n"
        )
        self.run_cmd(
            "configure set profile.dev.s3.signature_version s3v4",
        )
        self.assertEqual(
            "[default]\n"
            "s3 =\n"
            "    signature_version = s3v4\n"
            "[profile dev]\n"
            "s3 =\n"
            "    signature_version = s3v4\n",
            self.get_config_file_contents(),
        )

    def test_override_existing_value(self):
        self.set_config_file_contents(
            "[default]\ns3 =\n    signature_version = v4\n"
        )
        self.run_cmd(
            "configure set default.s3.signature_version NEWVALUE",
        )
        self.assertEqual(
            "[default]\ns3 =\n    signature_version = NEWVALUE\n",
            self.get_config_file_contents(),
        )

    def test_get_nested_attribute(self):
        self.set_config_file_contents(
            "[default]\ns3 =\n    signature_version = v4\n"
        )
        stdout, _, _ = self.run_cmd(
            "configure get default.s3.signature_version"
        )

        self.assertEqual(stdout.strip(), "v4")
        stdout, _, _ = self.run_cmd(
            "configure get default.bad.doesnotexist", expected_rc=1
        )
        self.assertEqual(stdout, "")

    def test_set_with_subsection_no_name_provided(self):
        _, stderr, _ = self.run_cmd(
            [
                "configure",
                "set",
                "--sso-session",
                '',
                "space",
                "test",
            ],
            expected_rc=252,
        )
        self.assertIn("Invalid value for --sso-session", stderr)

    def test_set_deeply_nested_property_in_subsection_results_in_error(self):
        self.set_config_file_contents(
            "[services my-services]\nec2 =\n    endpoint_url = localhost\n"
        )

        _, stderr, _ = self.run_cmd(
            [
                "configure",
                "set",
                "--services",
                'my-services',
                "s3.express.endpoint_url",
                "localhost",
            ],
            expected_rc=252,
        )
        self.assertIn(
            "Found more than two parts in the property to set.", stderr
        )

    def test_set_with_two_subsections_specified_results_in_error(self):
        _, stderr, _ = self.run_cmd(
            [
                "configure",
                "set",
                "--sso-session",
                'my-sso',
                "--services",
                'my-services',
                "space",
                "test",
            ],
            expected_rc=252,
        )
        self.assertIn(
            "cannot be specified when one of the following keys are also specified:",
            stderr,
        )

    def test_set_updates_existing_property_in_subsection(self):
        self.set_config_file_contents(
            "[sso-session my-sso-session]\nsso_region = us-west-2\n"
        )

        self.run_cmd(
            [
                "configure",
                "set",
                "--sso-session",
                'my-sso-session',
                "sso_region",
                "eu-central-1",
            ],
            expected_rc=0,
        )
        self.assertEqual(
            "[sso-session my-sso-session]\nsso_region = eu-central-1\n",
            self.get_config_file_contents(),
        )
    def test_set_rejects_newline_in_value(self):
        _, stderr, _ = self.run_cmd(
            ["configure", "set", "region", "us-east-1\nus-west-2"],
            expected_rc=255,
        )
        self.assertIn("newline", stderr)
        # To avoid leaking sensitive values,
        # values should not appear in stderr.
        self.assertNotIn("us-east-1\nus-west-2", stderr)

    def test_set_rejects_carriage_return_in_value(self):
        _, stderr, _ = self.run_cmd(
            ["configure", "set", "region", "us-east-1\rus-west-2"],
            expected_rc=255,
        )
        self.assertIn("newline", stderr)
        # To avoid leaking sensitive values,
        # values should not appear in stderr.
        self.assertNotIn("us-east-1\rus-west-2", stderr)

    def test_set_rejects_newline_in_nested_value(self):
        _, stderr, _ = self.run_cmd(
            ["configure", "set", "default.s3.signature_version", "s3v4\nfoo"],
            expected_rc=255,
        )
        self.assertIn("newline", stderr)
        # To avoid leaking sensitive values,
        # values should not appear in stderr.
        self.assertNotIn("s3v4\nfoo", stderr)

    def test_newline_injection_does_not_write_injected_key_to_file(self):
        # Simulates: aws configure set output $'table\nregion = us-east-1'
        # The injected key must not appear anywhere in the config file.
        self.set_config_file_contents("[default]\n")
        self.run_cmd(
            ["configure", "set", "output", "table\nregion = us-east-1"],
            expected_rc=255,
        )
        contents = self.get_config_file_contents()
        self.assertNotIn("region", contents)

    def test_newline_injection_does_not_set_injected_key_in_parsed_config(self):
        # Even if the file were somehow written, the injected key must not be
        # readable back via 'configure get'.
        self.set_config_file_contents("[default]\n")
        self.run_cmd(
            ["configure", "set", "output", "table\nregion = us-east-1"],
            expected_rc=255,
        )
        # Re-create the driver so it re-reads the (unchanged) config file.
        self.driver = create_clidriver()
        stdout, _, _ = self.run_cmd(
            "configure get region", expected_rc=1
        )
        self.assertEqual(stdout.strip(), "")


class TestConfigureHasArgTable(unittest.TestCase):
    def test_configure_command_has_arg_table(self):
        m = mock.Mock()
        command = ConfigureCommand(m)
        self.assertEqual(command.arg_table, {})



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.autocomplete import filters
from awscli.autocomplete.completer import CompletionResult
from awscli.testutils import mock, unittest


class TestFilters(unittest.TestCase):
    def setUp(self):
        self.completions = [
            CompletionResult('version'),
            CompletionResult('foo', display_text='Help text'),
            CompletionResult('various text'),
        ]

    def _assert_filter_response_as_expected(
        self, response_filter, user_input, expected_output
    ):
        output = response_filter(user_input, self.completions)
        self.assertEqual(output, expected_output)

    def test_fuzzy_filter(self):
        self._assert_filter_response_as_expected(
            filters.fuzzy_filter,
            'ro',
            [CompletionResult('various text'), CompletionResult('version')],
        )
        self._assert_filter_response_as_expected(
            filters.fuzzy_filter,
            'et',
            [
                CompletionResult('various text'),
                CompletionResult('foo', display_text='Help text'),
            ],
        )
        self._assert_filter_response_as_expected(
            filters.fuzzy_filter,
            'vs',
            [
                CompletionResult('version'),
                CompletionResult('various text'),
            ],
        )

    def test_startswith_filter(self):
        self._assert_filter_response_as_expected(
            filters.startswith_filter,
            'He',
            [
                CompletionResult('foo', display_text='Help text'),
            ],
        )
        self._assert_filter_response_as_expected(
            filters.startswith_filter,
            'v',
            [
                CompletionResult('version'),
                CompletionResult('various text'),
            ],
        )



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from tests.unit.customizations.emr import (
    EMRBaseAWSCommandParamsTest as BaseAWSCommandParamsTest,
)


class TestTerminateClusters(BaseAWSCommandParamsTest):
    prefix = 'emr terminate-clusters'

    def test_cluster_id(self):
        args = ' --cluster-ids j-ABC123456'
        cmdline = self.prefix + args
        result = {'JobFlowIds': ['j-ABC123456']}
        self.assert_params_for_cmd(cmdline, result)

    def test_cluster_ids(self):
        args = ' --cluster-ids j-ABC123456 j-AAAAAAA'
        cmdline = self.prefix + args
        result = {'JobFlowIds': ['j-ABC123456', 'j-AAAAAAA']}
        self.assert_params_for_cmd(cmdline, result)


if __name__ == "__main__":
    unittest.main()



# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import pytest
from botocore.model import OperationModel

from awscli.clidriver import create_clidriver


def _generate_command_tests():
    driver = create_clidriver()
    help_command = driver.create_help_command()
    top_level_params = set(driver.create_help_command().arg_table.keys())
    for command_name, command_obj in list(help_command.command_table.items()):
        sub_help = command_obj.create_help_command()
        if hasattr(sub_help, 'command_table'):
            for sub_name, sub_command in sub_help.command_table.items():
                op_help = sub_command.create_help_command()
                model = op_help.obj
                arg_table = op_help.arg_table
                if not isinstance(model, OperationModel):
                    continue
                yield (
                    command_name,
                    sub_name,
                    model.service_model.service_name,
                    model.name,
                    arg_table,
                    top_level_params
                )


@pytest.mark.validates_models
@pytest.mark.parametrize(
    "command_name, sub_name, service_name, "
    "operation_name, arg_table, builtins",
    _generate_command_tests()
)
def test_no_shadowed_builtins(
        command_name,
        sub_name,
        service_name,
        operation_name,
        arg_table,
        builtins,
        record_property
):
    """Verify no command params are shadowed or prefixed by the built-in param.

    The CLI parses all command line options into a single namespace.
    This means that option names must be unique and cannot conflict
    with the top level params.

    For example, there's a top level param ``--version``.  If an
    operation for a service also provides a ``--version`` option,
    it can never be called because we'll assume the user meant
    the top level ``--version`` param.

    Beyond just direct shadowing, a param which prefixes a builtin
    is also effectively shadowed because argparse will expand
    prefixes of arguments. So `--end` would expand to `--endpoint-url`
    for instance.

    In order to ensure this doesn't happen, this test will go
    through every command table and ensure we're not shadowing
    any builtins.

    Also, rather than being a test generator, we're going to just
    aggregate all the failures in one pass and surface them as
    a single test failure.

    """
    errors = []
    for arg_name in arg_table:
        if any(p.startswith(arg_name) for p in builtins):
            # Then we are shadowing or prefixing a top level argument
            errors.append(
                'Shadowing/Prefixing a top level option: '
                f'{command_name}.{sub_name}.{arg_name}'
            )
    if errors:
        # Store the service and operation in
        # PyTest custom properties
        record_property(
            'aws_service', service_name
        )
        record_property('aws_operation', operation_name)
        raise AssertionError('\n' + '\n'.join(errors))



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import json

from awscli.testutils import mock
from tests.unit.customizations.emr import (
    EMRBaseAWSCommandParamsTest as BaseAWSCommandParamsTest,
)
from tests.unit.customizations.emr import test_constants as CONSTANTS

INSTANCE_GROUPS_WITH_AUTOSCALING_POLICY = (
    ' InstanceGroupType=TASK,InstanceType=d2.xlarge,InstanceCount=2,'
    'AutoScalingPolicy={Constraints={MinCapacity=1,MaxCapacity=2},'
    'Rules=[{Name=TestRule,Description=TestDescription,Action={Market=ON_DEMAND,'
    'SimpleScalingPolicyConfiguration={AdjustmentType=EXACT_CAPACITY,ScalingAdjustment=2,CoolDown=5}},'
    'Trigger={CloudWatchAlarmDefinition={ComparisonOperator=GREATER_THAN,'
    'EvaluationPeriods=5,MetricName=TestMetric,Namespace=EMR,Period=3,Statistic=MAXIMUM,'
    'Threshold=4.565,Unit=NONE,Dimensions=[{Key=TestKey,Value=TestValue}]}}}]}'
)

INSTANCE_GROUPS_WITH_EBS_VOLUME_ARG = ' InstanceGroupType=TASK,InstanceType=d2.xlarge,InstanceCount=2,EbsConfiguration={EbsOptimized=true,EbsBlockDeviceConfigs=[{VolumeSpecification={VolumeType=gp2,SizeInGB=100,Iops=100},VolumesPerInstance=4},{VolumeSpecification={VolumeType=gp2,SizeInGB=100,Iops=100}}]}'

INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_VOLTYPE_ARG = ' InstanceGroupType=TASK,InstanceType=d2.xlarge,InstanceCount=2,EbsConfiguration={EbsOptimized=true,EbsBlockDeviceConfigs=[{VolumeSpecification={SizeInGB=100,Iops=100},VolumesPerInstance=4},{VolumeSpecification={VolumeType=gp2,SizeInGB=100,Iops=100}}]}'

INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_SIZE_ARG = ' InstanceGroupType=TASK,InstanceType=d2.xlarge,InstanceCount=2,EbsConfiguration={EbsOptimized=true,EbsBlockDeviceConfigs=[{VolumeSpecification={VolumeType=gp2,Iops=100},VolumesPerInstance=4},{VolumeSpecification={VolumeType=gp2,SizeInGB=100,Iops=100}}]}'

INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_VOLSPEC_ARG = ' InstanceGroupType=TASK,InstanceType=d2.xlarge,InstanceCount=2,EbsConfiguration={EbsOptimized=true}'

INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_IOPS_ARG = ' InstanceGroupType=TASK,InstanceType=d2.xlarge,InstanceCount=2,EbsConfiguration={EbsOptimized=true,EbsBlockDeviceConfigs=[{VolumeSpecification={VolumeType=gp2,SizeInGB=100},VolumesPerInstance=4}]}'

MULTIPLE_INSTANCE_GROUPS_WITH_EBS_VOLUMES_VOLUME_ARG = ' InstanceGroupType=TASK,InstanceType=d2.xlarge,InstanceCount=2,EbsConfiguration={EbsOptimized=true,EbsBlockDeviceConfigs=[{VolumeSpecification={VolumeType=gp2,SizeInGB=100},VolumesPerInstance=4}]} InstanceGroupType=CORE,InstanceType=d2.xlarge,InstanceCount=2,EbsConfiguration={EbsOptimized=true,EbsBlockDeviceConfigs=[{VolumeSpecification={VolumeType=gp2,SizeInGB=100,Iops=20}},{VolumeSpecification={VolumeType=gp2,SizeInGB=100,Iops=40}}]}'

INSTANCE_GROUPS_WITH_CUSTOM_AMI_ARG = ' InstanceGroupType=TASK,InstanceType=d2.xlarge,InstanceCount=2,CustomAmiId=ami-deadbeef'

DEFAULT_INSTANCE_GROUPS = [
    {
        'InstanceRole': 'TASK',
        'InstanceCount': 10,
        'Name': 'TASK',
        'Market': 'ON_DEMAND',
        'InstanceType': 'm2.large',
    }
]


DEFAULT_INSTANCE_GROUPS_WITH_CUSTOM_AMI = [
    {
        'CustomAmiId': 'ami-deadbeef',
        'InstanceRole': 'TASK',
        'InstanceCount': 2,
        'Name': 'TASK',
        'Market': 'ON_DEMAND',
        'InstanceType': 'd2.xlarge',
    }
]

DEFAULT_INSTANCE_GROUPS_WITH_EBS_CONFIG = [
    {
        'EbsConfiguration': {
            'EbsOptimized': True,
            'EbsBlockDeviceConfigs': [
                {
                    'VolumeSpecification': {
                        'Iops': 100,
                        'SizeInGB': 100,
                        'VolumeType': 'gp2',
                    },
                    'VolumesPerInstance': 4,
                },
                {
                    'VolumeSpecification': {
                        'Iops': 100,
                        'SizeInGB': 100,
                        'VolumeType': 'gp2',
                    }
                },
            ],
        },
        'InstanceCount': 2,
        'InstanceRole': 'TASK',
        'InstanceType': 'd2.xlarge',
        'Market': 'ON_DEMAND',
        'Name': 'TASK',
    }
]

DEFAULT_INSTANCE_GROUPS_WITH_EBS_CONFIG_MISSING_IOPS = [
    {
        'EbsConfiguration': {
            'EbsOptimized': True,
            'EbsBlockDeviceConfigs': [
                {
                    'VolumeSpecification': {
                        'SizeInGB': 100,
                        'VolumeType': 'gp2',
                    },
                    'VolumesPerInstance': 4,
                }
            ],
        },
        'InstanceCount': 2,
        'InstanceRole': 'TASK',
        'InstanceType': 'd2.xlarge',
        'Market': 'ON_DEMAND',
        'Name': 'TASK',
    }
]

DEFAULT_INSTANCE_GROUPS_WITH_EBS_CONFIG_MISSING_VOLSPEC = [
    {
        'EbsConfiguration': {'EbsOptimized': True},
        'InstanceCount': 2,
        'InstanceRole': 'TASK',
        'InstanceType': 'd2.xlarge',
        'Market': 'ON_DEMAND',
        'Name': 'TASK',
    }
]

DEFAULT_INSTANCE_GROUPS_WITH_AUTOSCALING_POLICY = [
    {
        'InstanceCount': 2,
        'InstanceRole': 'TASK',
        'InstanceType': 'd2.xlarge',
        'Market': 'ON_DEMAND',
        'Name': 'TASK',
        'AutoScalingPolicy': {
            'Constraints': {'MinCapacity': 1, 'MaxCapacity': 2},
            'Rules': [
                {
                    'Name': 'TestRule',
                    'Description': 'TestDescription',
                    'Action': {
                        'Market': 'ON_DEMAND',
                        'SimpleScalingPolicyConfiguration': {
                            'AdjustmentType': 'EXACT_CAPACITY',
                            'ScalingAdjustment': 2,
                            'CoolDown': 5,
                        },
                    },
                    'Trigger': {
                        'CloudWatchAlarmDefinition': {
                            'ComparisonOperator': 'GREATER_THAN',
                            'Dimensions': [
                                {'Key': 'TestKey', 'Value': 'TestValue'}
                            ],
                            'EvaluationPeriods': 5,
                            'MetricName': 'TestMetric',
                            'Namespace': 'EMR',
                            'Period': 3,
                            'Statistic': 'MAXIMUM',
                            'Threshold': 4.565,
                            'Unit': 'NONE',
                        }
                    },
                }
            ],
        },
    }
]

DEFAULT_MULTIPLE_INSTANCE_GROUPS_WITH_EBS_CONFIG = [
    {
        'EbsConfiguration': {
            'EbsOptimized': True,
            'EbsBlockDeviceConfigs': [
                {
                    'VolumeSpecification': {
                        'SizeInGB': 100,
                        'VolumeType': 'gp2',
                    },
                    'VolumesPerInstance': 4,
                }
            ],
        },
        'InstanceCount': 2,
        'InstanceRole': 'TASK',
        'InstanceType': 'd2.xlarge',
        'Market': 'ON_DEMAND',
        'Name': 'TASK',
    },
    {
        'EbsConfiguration': {
            'EbsOptimized': True,
            'EbsBlockDeviceConfigs': [
                {
                    'VolumeSpecification': {
                        'Iops': 20,
                        'SizeInGB': 100,
                        'VolumeType': 'gp2',
                    }
                },
                {
                    'VolumeSpecification': {
                        'Iops': 40,
                        'SizeInGB': 100,
                        'VolumeType': 'gp2',
                    }
                },
            ],
        },
        'InstanceCount': 2,
        'InstanceRole': 'CORE',
        'InstanceType': 'd2.xlarge',
        'Market': 'ON_DEMAND',
        'Name': 'CORE',
    },
]


ADD_INSTANCE_GROUPS_RESULT = {
    "InstanceGroupIds": ["ig-XXXX"],
    "ClusterArn": "arn:aws:elasticmapreduce:region:012345678910:cluster/j-XXXX",
    "JobFlowId": "j-YYYY",
}

CONSTRUCTED_RESULT = {
    "InstanceGroupIds": ["ig-XXXX"],
    "ClusterArn": "arn:aws:elasticmapreduce:region:012345678910:cluster/j-XXXX",
    "ClusterId": "j-YYYY",
}


class TestAddInstanceGroups(BaseAWSCommandParamsTest):
    prefix = 'emr add-instance-groups --cluster-id J-ABCD --instance-groups'

    def test_instance_groups_with_autoscaling_policy(self):
        cmd = self.prefix
        cmd += INSTANCE_GROUPS_WITH_AUTOSCALING_POLICY
        result = {
            'JobFlowId': 'J-ABCD',
            'InstanceGroups': DEFAULT_INSTANCE_GROUPS_WITH_AUTOSCALING_POLICY,
        }
        self.assert_params_for_cmd(cmd, result)

    def assert_error_message_has_field_name(self, error_msg, field_name):
        self.assertIn('Missing required parameter', error_msg)
        self.assertIn(field_name, error_msg)

    def test_instance_groups_default_name_market(self):
        cmd = self.prefix
        cmd += ' InstanceGroupType=TASK,InstanceCount=10,InstanceType=m2.large'
        result = {
            'JobFlowId': 'J-ABCD',
            'InstanceGroups': DEFAULT_INSTANCE_GROUPS,
        }

        self.assert_params_for_cmd(cmd, result)

    def test_instance_groups_missing_instance_group_type_error(self):
        cmd = (
            self.prefix
            + ' Name=Task,InstanceType=m1.small,'
            + 'InstanceCount=5'
        )
        result = self.run_cmd(cmd, 252)
        self.assert_error_message_has_field_name(
            result[1], 'InstanceGroupType'
        )

    def test_instance_groups_missing_instance_type_error(self):
        cmd = (
            self.prefix
            + ' Name=Task,InstanceGroupType=Task,'
            + 'InstanceCount=5'
        )
        stderr = self.run_cmd(cmd, 252)[1]
        self.assert_error_message_has_field_name(stderr, 'InstanceType')

    def test_instance_groups_missing_instance_count_error(self):
        cmd = (
            self.prefix
            + ' Name=Task,InstanceGroupType=Task,'
            + 'InstanceType=m1.xlarge'
        )
        stderr = self.run_cmd(cmd, 252)[1]
        self.assert_error_message_has_field_name(stderr, 'InstanceCount')

    def test_instance_groups_all_fields(self):
        cmd = (
            self.prefix
            + ' InstanceGroupType=MASTER,Name="MasterGroup",'
            + 'InstanceCount=1,InstanceType=m1.large'
        )
        cmd += (
            ' InstanceGroupType=CORE,Name="CoreGroup",InstanceCount=1,'
            + 'InstanceType=m1.xlarge,BidPrice=1.234'
        )
        cmd += (
            ' InstanceGroupType=TASK,Name="TaskGroup",InstanceCount=2,'
            + 'InstanceType=m1.large'
        )

        expected_instance_groups = [
            {
                'InstanceRole': 'MASTER',
                'InstanceCount': 1,
                'Name': 'MasterGroup',
                'Market': 'ON_DEMAND',
                'InstanceType': 'm1.large',
            },
            {
                'InstanceRole': 'CORE',
                'InstanceCount': 1,
                'Name': 'CoreGroup',
                'Market': 'SPOT',
                'BidPrice': '1.234',
                'InstanceType': 'm1.xlarge',
            },
            {
                'InstanceRole': 'TASK',
                'InstanceCount': 2,
                'Name': 'TaskGroup',
                'Market': 'ON_DEMAND',
                'InstanceType': 'm1.large',
            },
        ]
        result = {
            'JobFlowId': 'J-ABCD',
            'InstanceGroups': expected_instance_groups,
        }

        self.assert_params_for_cmd(cmd, result)

    def test_instance_groups_spot_bidprice_equals_ondemandprice(self):
        cmd = (
            self.prefix
            + ' InstanceGroupType=MASTER,Name="MasterGroup",'
            + 'InstanceCount=1,InstanceType=m1.large,BidPrice=OnDemandPrice'
        )
        cmd += (
            ' InstanceGroupType=CORE,Name="CoreGroup",InstanceCount=1,'
            + 'InstanceType=m1.xlarge,BidPrice=OnDemandPrice'
        )
        cmd += (
            ' InstanceGroupType=TASK,Name="TaskGroup",InstanceCount=2,'
            + 'InstanceType=m1.large,BidPrice=OnDemandPrice'
        )

        expected_instance_groups = [
            {
                'InstanceRole': 'MASTER',
                'InstanceCount': 1,
                'Name': 'MasterGroup',
                'Market': 'SPOT',
                'InstanceType': 'm1.large',
            },
            {
                'InstanceRole': 'CORE',
                'InstanceCount': 1,
                'Name': 'CoreGroup',
                'Market': 'SPOT',
                'InstanceType': 'm1.xlarge',
            },
            {
                'InstanceRole': 'TASK',
                'InstanceCount': 2,
                'Name': 'TaskGroup',
                'Market': 'SPOT',
                'InstanceType': 'm1.large',
            },
        ]
        result = {
            'JobFlowId': 'J-ABCD',
            'InstanceGroups': expected_instance_groups,
        }

        self.assert_params_for_cmd(cmd, result)

    def test_instance_groups_with_ebs_config(self):
        cmd = self.prefix
        cmd += INSTANCE_GROUPS_WITH_EBS_VOLUME_ARG
        result = {
            'JobFlowId': 'J-ABCD',
            'InstanceGroups': DEFAULT_INSTANCE_GROUPS_WITH_EBS_CONFIG,
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_groups_with_ebs_config_missing_volume_type(self):
        cmd = self.prefix
        cmd += INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_VOLTYPE_ARG
        stderr = self.run_cmd(cmd, 252)[1]
        self.assert_error_message_has_field_name(stderr, 'VolumeType')

    def test_instance_groups_with_ebs_config_missing_size(self):
        cmd = self.prefix
        cmd += INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_SIZE_ARG
        stderr = self.run_cmd(cmd, 252)[1]
        self.assert_error_message_has_field_name(stderr, 'SizeInGB')

    def test_instance_groups_with_ebs_config_missing_volume_spec(self):
        cmd = self.prefix
        cmd += INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_VOLSPEC_ARG
        result = {
            'JobFlowId': 'J-ABCD',
            'InstanceGroups': DEFAULT_INSTANCE_GROUPS_WITH_EBS_CONFIG_MISSING_VOLSPEC,
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_groups_with_ebs_config_missing_iops(self):
        cmd = self.prefix
        cmd += INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_IOPS_ARG
        result = {
            'JobFlowId': 'J-ABCD',
            'InstanceGroups': DEFAULT_INSTANCE_GROUPS_WITH_EBS_CONFIG_MISSING_IOPS,
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_groups_with_ebs_config_multiple_instance_groups(self):
        cmd = self.prefix
        cmd += MULTIPLE_INSTANCE_GROUPS_WITH_EBS_VOLUMES_VOLUME_ARG
        result = {
            'JobFlowId': 'J-ABCD',
            'InstanceGroups': DEFAULT_MULTIPLE_INSTANCE_GROUPS_WITH_EBS_CONFIG,
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_groups_with_custom_ami_instance_groups(self):
        cmd = self.prefix
        cmd += INSTANCE_GROUPS_WITH_CUSTOM_AMI_ARG
        result = {
            'JobFlowId': 'J-ABCD',
            'InstanceGroups': DEFAULT_INSTANCE_GROUPS_WITH_CUSTOM_AMI,
        }
        self.assert_params_for_cmd(cmd, result)

    @mock.patch('awscli.customizations.emr.emrutils.call')
    def test_constructed_result(self, call_patch):
        call_patch.return_value = ADD_INSTANCE_GROUPS_RESULT
        cmd = self.prefix
        cmd += ' InstanceGroupType=TASK,InstanceCount=10,InstanceType=m2.large'

        result = self.run_cmd(cmd, expected_rc=0)
        result_json = json.loads(result[0])

        self.assertEqual(result_json, CONSTRUCTED_RESULT)


if __name__ == "__main__":
    unittest.main()
